@@ -1,21 +1,14 @@
 /*
-   Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -40,7 +33,7 @@
 #include <my_dir.h>
 #include <mysql_version.h>
 
-#define MAX_ROWS  1000
+#define MAX_ROWS  2000
 #define HEADER_LENGTH 32                /* Length of header in errmsg.sys */
 #define ERRMSG_VERSION 3                /* Version number of errmsg.sys */
 #define DEFAULT_CHARSET_DIR "../sql/share/charsets"
@@ -190,6 +183,12 @@ int main(int argc, char *argv[])
       fprintf(stderr, "Failed to parse input file %s\n", TXTFILE);
       DBUG_RETURN(1);
     }
+    if (row_count > MAX_ROWS)
+    {
+      fprintf(stderr, "Found too many error messages. ");
+      fprintf(stderr, "Increase MAX_ROWS in extra/comp_err.c.\n");
+      DBUG_RETURN(1);
+    }
 #if MYSQL_VERSION_ID >= 50100 && MYSQL_VERSION_ID < 50500
 /* Number of error messages in 5.1 - do not change this number! */
 #define MYSQL_OLD_GA_ERROR_MESSAGE_COUNT 641
@@ -253,7 +252,6 @@ static void print_escaped_string(FILE *f, const char *str)
 
 static int create_header_files(struct errors *error_head)
 {
-  uint er_last= 0;
   FILE *er_definef, *sql_statef, *er_namef;
   struct errors *tmp_error;
   struct message *er_msg;
@@ -281,7 +279,40 @@ static int create_header_files(struct errors *error_head)
   fprintf(sql_statef, "/* Autogenerated file, please don't edit */\n\n");
   fprintf(er_namef, "/* Autogenerated file, please don't edit */\n\n");
 
-  fprintf(er_definef, "#define ER_ERROR_FIRST %d\n", error_head->d_code);
+  fprintf(er_definef, "#ifndef MYSQLD_ERROR_INCLUDED\n");
+  fprintf(er_definef, "#define MYSQLD_ERROR_INCLUDED\n\n");
+
+  /*
+    Find out how many sections of error messages we have, what the first
+    number in each section is and the number of messages in each section.
+  */
+  {
+    int error_code= error_head->d_code;
+    int section_size[100]; /* Assume that 100 sections is enough. */
+    int current_section= 0;
+    int section_nr;
+    section_size[0]= 0;
+    fprintf(er_definef,
+            "static const int errmsg_section_start[] = { %d", error_code);
+    for (tmp_error= error_head; tmp_error; tmp_error= tmp_error->next_error)
+    {
+      if (tmp_error->d_code != error_code) /* Starting new section? */
+      {
+        fprintf(er_definef, ", %d", tmp_error->d_code);
+        error_code= tmp_error->d_code;
+        section_size[++current_section]= 0;
+      }
+      error_code++;
+      section_size[current_section]++;
+    }
+    fprintf(er_definef, " };\n");
+
+    fprintf(er_definef,
+            "static const int errmsg_section_size[] = { %d", section_size[0]);
+    for (section_nr= 1; section_nr <= current_section; section_nr++)
+      fprintf(er_definef, ", %d", section_size[section_nr]);
+    fprintf(er_definef, " };\n\n");
+  }
 
   for (tmp_error= error_head; tmp_error; tmp_error= tmp_error->next_error)
   {
@@ -291,7 +322,6 @@ static int create_header_files(struct errors *error_head)
     */
     fprintf(er_definef, "#define %s %d\n", tmp_error->er_name,
 	    tmp_error->d_code);
-    er_last= tmp_error->d_code;
 
     /* generating sql_state.h file */
     if (tmp_error->sql_code1[0] || tmp_error->sql_code2[0])
@@ -307,7 +337,7 @@ static int create_header_files(struct errors *error_head)
     fprintf(er_namef, "\" },\n");
   }
   /* finishing off with mysqld_error.h */
-  fprintf(er_definef, "#define ER_ERROR_LAST %d\n", er_last);
+  fprintf(er_definef, "#endif\n");
   my_fclose(er_definef, MYF(0));
   my_fclose(sql_statef, MYF(0));
   my_fclose(er_namef, MYF(0));
@@ -388,7 +418,7 @@ static int create_sys_files(struct languages *lang_head,
     /* continue with header of the errmsg.sys file */
     length= ftell(to) - HEADER_LENGTH - row_count * 4;
     memset(head, 0, HEADER_LENGTH);
-    bmove((uchar *) head, (uchar *) file_head, 4);
+    memmove(head, file_head, 4);
     head[4]= 1;
     int4store(head + 6, length);
     int4store(head + 10, row_count);
@@ -461,7 +491,8 @@ static int parse_input_file(const char *file_name, struct errors **top_error,
   char *str, buff[1000];
   struct errors *current_error= 0, **tail_error= top_error;
   struct message current_message;
-  int rcount= 0;
+  int rcount= 0; /* Number of error codes in current section. */
+  int ecount= 0; /* Number of error codes in total. */
   DBUG_ENTER("parse_input_file");
 
   *top_error= 0;
@@ -487,6 +518,7 @@ static int parse_input_file(const char *file_name, struct errors **top_error,
 	fprintf(stderr, "Failed to parse the error offset string!\n");
 	DBUG_RETURN(0);
       }
+      rcount= 0; /* Reset count if a fixed number is set. */
       continue;
     }
     if (is_prefix(str, "default-language"))
@@ -540,7 +572,8 @@ static int parse_input_file(const char *file_name, struct errors **top_error,
 	fprintf(stderr, "Failed to parse the error name string\n");
 	DBUG_RETURN(0);
       }
-      rcount++;                         /* Count number of unique errors */
+      rcount++;
+      ecount++;                         /* Count number of unique errors */
 
       /* add error to the list */
       *tail_error= current_error;
@@ -556,7 +589,7 @@ static int parse_input_file(const char *file_name, struct errors **top_error,
   *tail_error= 0;				/* Mark end of list */
 
   my_fclose(file, MYF(0));
-  DBUG_RETURN(rcount);
+  DBUG_RETURN(ecount);
 }
 
 
@@ -819,8 +852,9 @@ static char *get_word(char **str)
   DBUG_ENTER("get_word");
 
   *str= find_end_of_word(start);
-  DBUG_RETURN(my_strndup(start, (uint) (*str - start),
-				    MYF(MY_WME | MY_FAE)));
+  DBUG_RETURN(my_strndup(PSI_NOT_INSTRUMENTED,
+                         start, (uint) (*str - start),
+			 MYF(MY_WME | MY_FAE)));
 }
 
 
@@ -853,8 +887,9 @@ static struct message *parse_message_string(struct message *new_message,
   while (*str != ' ' && *str != '\t' && *str)
     str++;
   if (!(new_message->lang_short_name=
-	my_strndup(start, (uint) (str - start),
-			      MYF(MY_WME | MY_FAE))))
+	my_strndup(PSI_NOT_INSTRUMENTED,
+                   start, (uint) (str - start),
+		   MYF(MY_WME | MY_FAE))))
     DBUG_RETURN(0);				/* Fatal error */
   DBUG_PRINT("info", ("msg_slang: %s", new_message->lang_short_name));
 
@@ -873,8 +908,9 @@ static struct message *parse_message_string(struct message *new_message,
   start= str + 1;
   str= parse_text_line(start);
 
-  if (!(new_message->text= my_strndup(start, (uint) (str - start),
-						 MYF(MY_WME | MY_FAE))))
+  if (!(new_message->text= my_strndup(PSI_NOT_INSTRUMENTED,
+                                      start, (uint) (str - start),
+                                      MYF(MY_WME | MY_FAE))))
     DBUG_RETURN(0);				/* Fatal error */
   DBUG_PRINT("info", ("msg_text: %s", new_message->text));
 
@@ -894,9 +930,11 @@ static struct errors *parse_error_string(char *str, int er_count)
   DBUG_PRINT("enter", ("str: %s", str));
 
   /* create a new element */
-  new_error= (struct errors *) my_malloc(sizeof(*new_error), MYF(MY_WME));
+  new_error= (struct errors *) my_malloc(PSI_NOT_INSTRUMENTED,
+                                         sizeof(*new_error), MYF(MY_WME));
 
-  if (my_init_dynamic_array(&new_error->msg, sizeof(struct message), 0, 0))
+  if (my_init_dynamic_array(&new_error->msg, sizeof(struct message),
+                            NULL, 0, 0))
     DBUG_RETURN(0);				/* OOM: Fatal error */
 
   /* getting the error name */
@@ -948,7 +986,7 @@ static struct errors *parse_error_string(char *str, int er_count)
   str= skip_delimiters(str);
   if (*str)
   {
-    fprintf(stderr, "The error line did not end with sql/odbc code!");
+    fprintf(stderr, "The error line did not end with sql/odbc code: '%s'\n", str);
     DBUG_RETURN(0);
   }
 
@@ -983,7 +1021,8 @@ static struct languages *parse_charset_string(char *str)
   do
   {
     /*creating new element of the linked list */
-    new_lang= (struct languages *) my_malloc(sizeof(*new_lang), MYF(MY_WME));
+    new_lang= (struct languages *) my_malloc(PSI_NOT_INSTRUMENTED,
+                                             sizeof(*new_lang), MYF(MY_WME));
     new_lang->next_lang= head;
     head= new_lang;
 
@@ -1030,8 +1069,8 @@ static void print_version(void)
 
 
 static my_bool
-get_one_option(int optid, const struct my_option *opt MY_ATTRIBUTE ((unused)),
-	       char *argument MY_ATTRIBUTE ((unused)))
+get_one_option(int optid, const struct my_option *opt __attribute__ ((unused)),
+	       char *argument __attribute__ ((unused)))
 {
   DBUG_ENTER("get_one_option");
   switch (optid) {
