@@ -1,20 +1,13 @@
--- Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
+-- Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
 --
 -- This program is free software; you can redistribute it and/or modify
--- it under the terms of the GNU General Public License, version 2.0,
--- as published by the Free Software Foundation.
---
--- This program is also distributed with certain software (including
--- but not limited to OpenSSL) that is licensed under separate terms,
--- as designated in a particular file or component or in included license
--- documentation.  The authors of MySQL hereby grant you an additional
--- permission to link the program and your derivative works with the
--- separately licensed software that they have included with MySQL.
+-- it under the terms of the GNU General Public License as published by
+-- the Free Software Foundation; version 2 of the License.
 --
 -- This program is distributed in the hope that it will be useful,
 -- but WITHOUT ANY WARRANTY; without even the implied warranty of
 -- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--- GNU General Public License, version 2.0, for more details.
+-- GNU General Public License for more details.
 --
 -- You should have received a copy of the GNU General Public License
 -- along with this program; if not, write to the Free Software Foundation,
@@ -106,6 +99,8 @@ INSERT INTO global_suppressions VALUES
  ("Forcing shutdown of [0-9]* plugins"),
  ("Forcing close of thread"),
 
+ ("innodb-page-size has been changed"),
+
  /*
    Due to timing issues, it might be that this warning
    is printed when the server shuts down and the
@@ -182,6 +177,8 @@ INSERT INTO global_suppressions VALUES
  ("No existing UUID has been found, so we assume that this is the first time that this server has been started.*"),
  /*It will print a warning if server is run without --explicit_defaults_for_timestamp.*/
  ("TIMESTAMP with implicit DEFAULT value is deprecated. Please use --explicit_defaults_for_timestamp server option (see documentation for more details)*"),
+ /*It will print a warning if a server is run without NO_AUTO_CREATE_USER sql mode.*/
+ ("'NO_AUTO_CREATE_USER' sql mode is not set. It will be made read-only in a future release."),
 
  /* Added 2009-08-XX after fixing Bug #42408 */
 
@@ -214,21 +211,14 @@ INSERT INTO global_suppressions VALUES
    Transient network failures that cause warnings on reconnect.
    BUG#47743 and BUG#47983.
  */
- ("Slave I/O: Get master SERVER_UUID failed with error:.*"),
- ("Slave I/O: Get master SERVER_ID failed with error:.*"),
- ("Slave I/O: Get master clock failed with error:.*"),
- ("Slave I/O: Get master COLLATION_SERVER failed with error:.*"),
- ("Slave I/O: Get master TIME_ZONE failed with error:.*"),
- ("Slave I/O: The slave I/O thread stops because a fatal error is encountered when it tried to SET @master_binlog_checksum on master.*"),
- ("Slave I/O: Get master BINLOG_CHECKSUM failed with error.*"),
- ("Slave I/O: Notifying master by SET @master_binlog_checksum= @@global.binlog_checksum failed with error.*"),
- /*
-   BUG#42147 - Concurrent DML and LOCK TABLE ... READ for InnoDB 
-   table cause warnings in errlog
-   Note: This is a temporary suppression until Bug#42147 can be 
-   fixed properly. See bug page for more information.
-  */
- ("Found lock of type 6 that is write and read locked"),
+ ("Slave I/O.*: Get master SERVER_UUID failed with error:.*"),
+ ("Slave I/O.*: Get master SERVER_ID failed with error:.*"),
+ ("Slave I/O.*: Get master clock failed with error:.*"),
+ ("Slave I/O.*: Get master COLLATION_SERVER failed with error:.*"),
+ ("Slave I/O.*: Get master TIME_ZONE failed with error:.*"),
+ ("Slave I/O.*: The slave I/O thread stops because a fatal error is encountered when it tried to SET @master_binlog_checksum on master.*"),
+ ("Slave I/O.*: Get master BINLOG_CHECKSUM failed with error.*"),
+ ("Slave I/O.*: Notifying master by SET @master_binlog_checksum= @@global.binlog_checksum failed with error.*"),
 
  /*
    Warning message is printed out whenever a slave is started with
@@ -246,7 +236,7 @@ INSERT INTO global_suppressions VALUES
   In MTS if the user issues a stop slave sql while it is scheduling a group
   of events, this warning is emitted.
   */
- ("Slave SQL: Coordinator thread of multi-threaded slave is being stopped in the middle of assigning a group of events.*"),
+ ("Slave SQL.*: Coordinator thread of multi-threaded slave is being stopped in the middle of assigning a group of events.*"),
  
  ("Changed limits: max_open_files: *"),
  ("Changed limits: max_connections: *"),
@@ -254,19 +244,14 @@ INSERT INTO global_suppressions VALUES
  ("Could not increase number of max_open_files to more than *"),
 
  /*
-  Warnings related to --secure-file-priv
+   Warning message introduced by wl#7706
  */
- ("Insecure configuration for --secure-file-priv:*"),
+ ("CA certificate .* is self signed"),
 
  /*
-  Bug#26585560, warning related to --pid-file
+   Warnings related to --secure-file-priv
  */
- ("Insecure configuration for --pid-file:*"),
- ("Few location(s) are inaccessible while checking PID filepath"),
- /*
-   Following WL#12670, this warning is expected.
- */
- ("Setting named_pipe_full_access_group='\\*everyone\\*' is insecure"),
+ ("Insecure configuration for --secure-file-priv:*"),
 
  ("THE_LAST_SUPPRESSION")||
 
