@@ -1,20 +1,14 @@
 /* Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation; version 2 of the
+   License.
 
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   This program is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+   General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -23,14 +17,22 @@
 
 #include "rpl_gtid.h"
 
-#include <ctype.h>
-#include <algorithm>
-#include "my_dbug.h"
-#include "mysqld_error.h"
-#include <algorithm>
+#include "my_stacktrace.h"       // my_safe_printf_stderr
+#include "mysqld_error.h"        // ER_*
+#include "sql_const.h"
+
+#ifndef MYSQL_CLIENT
+#include "log.h"                 // sql_print_warning
+#endif
+
+PSI_memory_key key_memory_Gtid_set_to_string;
+PSI_memory_key key_memory_Gtid_set_Interval_chunk;
 
 using std::min;
 using std::max;
+using std::list;
+
+#define MAX_NEW_TRUNK_ALLOCATE_TRIES 10
 
 const Gtid_set::String_format Gtid_set::default_string_format=
 {
@@ -53,33 +55,54 @@ const Gtid_set::String_format Gtid_set::commented_string_format=
 };
 
 
-Gtid_set::Gtid_set(Sid_map *_sid_map, Checkable_rwlock *_sid_lock)
-  : sid_lock(_sid_lock), sid_map(_sid_map)
+Gtid_set::Gtid_set(Sid_map *_sid_map, Checkable_rwlock *_sid_lock
+#ifdef HAVE_PSI_INTERFACE
+                   ,PSI_mutex_key free_intervals_mutex_key
+#endif
+                  )
+  : sid_lock(_sid_lock), sid_map(_sid_map),
+    m_intervals(key_memory_Gtid_set_Interval_chunk)
 {
-  init();
+  init(
+#ifdef HAVE_PSI_INTERFACE
+       free_intervals_mutex_key
+#endif
+      );
 }
 
 
 Gtid_set::Gtid_set(Sid_map *_sid_map, const char *text,
-                   enum_return_status *status, Checkable_rwlock *_sid_lock)
-  : sid_lock(_sid_lock), sid_map(_sid_map)
+                   enum_return_status *status, Checkable_rwlock *_sid_lock
+#ifdef HAVE_PSI_INTERFACE
+                   ,PSI_mutex_key free_intervals_mutex_key
+#endif
+                  )
+  : sid_lock(_sid_lock), sid_map(_sid_map),
+    m_intervals(key_memory_Gtid_set_Interval_chunk)
 {
   DBUG_ASSERT(_sid_map != NULL);
-  init();
+  init(
+#ifdef HAVE_PSI_INTERFACE
+       free_intervals_mutex_key
+#endif
+      );
   *status= add_gtid_text(text);
 }
 
 
-void Gtid_set::init()
+void Gtid_set::init(
+#ifdef HAVE_PSI_INTERFACE
+                    PSI_mutex_key free_intervals_mutex_key
+#endif
+                   )
 {
   DBUG_ENTER("Gtid_set::init");
   cached_string_length= -1;
   cached_string_format= NULL;
   chunks= NULL;
   free_intervals= NULL;
-  my_init_dynamic_array(&intervals, sizeof(Interval *), 0, 8);
   if (sid_lock)
-    mysql_mutex_init(0, &free_intervals_mutex, NULL);
+    mysql_mutex_init(free_intervals_mutex_key, &free_intervals_mutex, NULL);
 #ifndef DBUG_OFF
   n_chunks= 0;
 #endif
@@ -94,14 +117,13 @@ Gtid_set::~Gtid_set()
   while (chunk != NULL)
   {
     Interval_chunk *next_chunk= chunk->next;
-    free(chunk);
+    my_free(chunk);
     chunk= next_chunk;
 #ifndef DBUG_OFF
     n_chunks--;
 #endif
   }
   DBUG_ASSERT(n_chunks == 0);
-  delete_dynamic(&intervals);
   if (sid_lock)
     mysql_mutex_destroy(&free_intervals_mutex);
   DBUG_VOID_RETURN;
@@ -146,12 +168,11 @@ enum_return_status Gtid_set::ensure_sidno(rpl_sidno sidno)
         }
       }
     }
-    if (allocate_dynamic(&intervals,
-                         sid_map == NULL ? sidno : sid_map->get_max_sidno()))
+    if (m_intervals.reserve(sid_map == NULL ? sidno : sid_map->get_max_sidno()))
       goto error;
     Interval *null_p= NULL;
     for (rpl_sidno i= max_sidno; i < sidno; i++)
-      if (insert_dynamic(&intervals, &null_p))
+      if (m_intervals.push_back(null_p))
         goto error;
     if (sid_lock != NULL)
     {
@@ -184,18 +205,53 @@ void Gtid_set::add_interval_memory_lock_taken(int n_ivs, Interval *ivs)
 }
 
 
-enum_return_status Gtid_set::create_new_chunk(int size)
+void Gtid_set::create_new_chunk(int size)
 {
   DBUG_ENTER("Gtid_set::create_new_chunk");
-  // allocate the new chunk. one element is already pre-allocated, so
-  // we only add size-1 elements to the size of the struct.
+  int i= 0;
+  Interval_chunk *new_chunk= NULL;
+
   assert_free_intervals_locked();
-  Interval_chunk *new_chunk=
-    (Interval_chunk *)my_malloc(sizeof(Interval_chunk) +
-                                sizeof(Interval) * (size - 1),
-                                MYF(MY_WME));
-  if (new_chunk == NULL)
-    RETURN_REPORTED_ERROR;
+  /*
+    Try to allocate the new chunk in MAX_NEW_TRUNK_ALLOCATE_TRIES
+    tries when encountering 'out of memory' situation.
+  */
+  while (i < MAX_NEW_TRUNK_ALLOCATE_TRIES)
+  {
+    /*
+      Allocate the new chunk. one element is already pre-allocated, so
+      we only add size-1 elements to the size of the struct.
+    */
+    new_chunk= (Interval_chunk *)my_malloc(key_memory_Gtid_set_Interval_chunk,
+                                           sizeof(Interval_chunk) +
+                                           sizeof(Interval) * (size - 1),
+                                           MYF(MY_WME));
+    if (new_chunk != NULL)
+    {
+#ifndef MYSQL_CLIENT
+      if (i > 0)
+        sql_print_warning("Server overcomes the temporary 'out of memory' "
+                          "in '%d' tries while allocating a new chunk of "
+                          "intervals for storing GTIDs.\n", i + 1);
+#endif
+      break;
+    }
+    /* Sleep 1 microsecond per try to avoid temporary 'out of memory' */
+    my_sleep(1);
+    i++;
+  }
+  /*
+    Terminate the server after failed to allocate the new chunk
+    in MAX_NEW_TRUNK_ALLOCATE_TRIES tries.
+  */
+  if (MAX_NEW_TRUNK_ALLOCATE_TRIES == i ||
+      DBUG_EVALUATE_IF("rpl_simulate_new_trunk_allocate_failure", 1, 0))
+  {
+    my_safe_print_system_time();
+    my_safe_printf_stderr("%s", "[Fatal] Out of memory while allocating "
+                          "a new chunk of intervals for storing GTIDs.\n");
+    _exit(MYSQLD_FAILURE_EXIT);
+  }
   // store the chunk in the list of chunks
   new_chunk->next= chunks;
   chunks= new_chunk;
@@ -204,20 +260,25 @@ enum_return_status Gtid_set::create_new_chunk(int size)
 #endif
   // add the intervals in the chunk to the list of free intervals
   add_interval_memory_lock_taken(size, new_chunk->intervals);
-  RETURN_OK;
+  DBUG_VOID_RETURN;
 }
 
 
-enum_return_status Gtid_set::get_free_interval(Interval **out)
+void Gtid_set::get_free_interval(Interval **out)
 {
   DBUG_ENTER("Gtid_set::get_free_interval");
   assert_free_intervals_locked();
   Interval_iterator ivit(this);
-  if (ivit.get() == NULL)
-    PROPAGATE_REPORTED_ERROR(create_new_chunk(CHUNK_GROW_SIZE));
+  bool simulate_failure=
+    DBUG_EVALUATE_IF("rpl_gtid_get_free_interval_simulate_out_of_memory",
+                     true, false);
+  if (simulate_failure)
+    DBUG_SET("+d,rpl_simulate_new_trunk_allocate_failure");
+  if (ivit.get() == NULL || simulate_failure)
+    create_new_chunk(CHUNK_GROW_SIZE);
   *out= ivit.get();
   ivit.set((*out)->next);
-  RETURN_OK;
+  DBUG_VOID_RETURN;
 }
 
 
@@ -263,7 +324,7 @@ void Gtid_set::clear()
 }
 
 
-enum_return_status
+void
 Gtid_set::add_gno_interval(Interval_iterator *ivitp,
                            rpl_gno start, rpl_gno end,
                            Free_intervals_lock *lock)
@@ -300,7 +361,7 @@ Gtid_set::add_gno_interval(Interval_iterator *ivitp,
       if (iv->end < end)
         iv->end= end;
       *ivitp= ivit;
-      RETURN_OK;
+      DBUG_VOID_RETURN;
     }
     ivit.next();
   }
@@ -312,18 +373,18 @@ Gtid_set::add_gno_interval(Interval_iterator *ivitp,
   */
   Interval *new_iv;
   lock->lock_if_not_locked();
-  PROPAGATE_REPORTED_ERROR(get_free_interval(&new_iv));
+  get_free_interval(&new_iv);
   new_iv->start= start;
   new_iv->end= end;
   ivit.insert(new_iv);
   *ivitp= ivit;
-  RETURN_OK;
+  DBUG_VOID_RETURN;
 }
 
 
-enum_return_status Gtid_set::remove_gno_interval(Interval_iterator *ivitp,
-                                                 rpl_gno start, rpl_gno end,
-                                                 Free_intervals_lock *lock)
+void Gtid_set::remove_gno_interval(Interval_iterator *ivitp,
+                                   rpl_gno start, rpl_gno end,
+                                   Free_intervals_lock *lock)
 {
   DBUG_ENTER("Gtid_set::remove_gno_interval(Interval_iterator *ivitp, rpl_gno start, rpl_gno end)");
   DBUG_ASSERT(start < end);
@@ -351,7 +412,7 @@ enum_return_status Gtid_set::remove_gno_interval(Interval_iterator *ivitp,
       // iv cuts also the end of the removed interval: split iv in two
       Interval *new_iv;
       lock->lock_if_not_locked();
-      PROPAGATE_REPORTED_ERROR(get_free_interval(&new_iv));
+      get_free_interval(&new_iv);
       new_iv->start= end;
       new_iv->end= iv->end;
       iv->end= start;
@@ -391,14 +452,14 @@ enum_return_status Gtid_set::remove_gno_interval(Interval_iterator *ivitp,
 
 ok:
   *ivitp= ivit;
-  RETURN_OK;
+  DBUG_VOID_RETURN;
 }
 
 
 rpl_gno parse_gno(const char **s)
 {
   char *endp;
-  rpl_gno ret= strtoll(*s, &endp, 0);
+  rpl_gno ret= my_strtoll(*s, &endp, 0);
   if (ret < 0 || ret == LLONG_MAX)
     return -1;
   *s= endp;
@@ -482,7 +543,7 @@ enum_return_status Gtid_set::add_gtid_text(const char *text, bool *anonymous)
         DBUG_PRINT("info", ("expected UUID; found garbage '%.80s' at char %d in '%s'", s, (int)(s - text), text));
         goto parse_error;
       }
-      s += rpl_sid::TEXT_LENGTH;
+      s += binary_log::Uuid::TEXT_LENGTH;
       rpl_sidno sidno= sid_map->add_sid(sid);
       if (sidno <= 0)
       {
@@ -536,10 +597,7 @@ enum_return_status Gtid_set::add_gtid_text(const char *text, bool *anonymous)
           Interval *current= ivit.get();
           if (current == NULL || start < current->start)
             ivit.init(this, sidno);
-          if (add_gno_interval(&ivit, start, end, &lock) != RETURN_STATUS_OK)
-          {
-            RETURN_REPORTED_ERROR;
-          }
+          add_gno_interval(&ivit, start, end, &lock);
         }
       }
     }
@@ -581,7 +639,7 @@ bool Gtid_set::is_valid(const char *text)
     // Parse SID.
     if (!rpl_sid::is_valid(s))
       DBUG_RETURN(false);
-    s += rpl_sid::TEXT_LENGTH;
+    s += binary_log::Uuid::TEXT_LENGTH;
     SKIP_WHITESPACE();
 
     // Iterate over intervals.
@@ -612,7 +670,7 @@ bool Gtid_set::is_valid(const char *text)
 }
 
 
-enum_return_status
+void
 Gtid_set::add_gno_intervals(rpl_sidno sidno,
                             Const_interval_iterator other_ivit,
                             Free_intervals_lock *lock)
@@ -623,16 +681,15 @@ Gtid_set::add_gno_intervals(rpl_sidno sidno,
   Interval_iterator ivit(this, sidno);
   while ((other_iv= other_ivit.get()) != NULL)
   {
-    PROPAGATE_REPORTED_ERROR(add_gno_interval(&ivit,
-                                              other_iv->start, other_iv->end,
-                                              lock));
+    add_gno_interval(&ivit, other_iv->start,
+                     other_iv->end, lock);
     other_ivit.next();
   }
-  RETURN_OK;
+  DBUG_VOID_RETURN;
 }
 
 
-enum_return_status
+void
 Gtid_set::remove_gno_intervals(rpl_sidno sidno,
                                Const_interval_iterator other_ivit,
                                Free_intervals_lock *lock)
@@ -643,17 +700,18 @@ Gtid_set::remove_gno_intervals(rpl_sidno sidno,
   Interval_iterator ivit(this, sidno);
   while ((other_iv= other_ivit.get()) != NULL)
   {
-    PROPAGATE_REPORTED_ERROR(remove_gno_interval(&ivit,
-                                                 other_iv->start, other_iv->end,
-                                                 lock));
+    remove_gno_interval(&ivit, other_iv->start, other_iv->end, lock);
     other_ivit.next();
   }
-  RETURN_OK;
+  DBUG_VOID_RETURN;
 }
 
 
 enum_return_status Gtid_set::add_gtid_set(const Gtid_set *other)
 {
+  /*
+    @todo refactor this and remove_gtid_set to avoid duplicated code
+  */
   DBUG_ENTER("Gtid_set::add_gtid_set(const Gtid_set *)");
   if (sid_lock != NULL)
     sid_lock->assert_some_wrlock();
@@ -663,19 +721,14 @@ enum_return_status Gtid_set::add_gtid_set(const Gtid_set *other)
   {
     PROPAGATE_REPORTED_ERROR(ensure_sidno(max_other_sidno));
     for (rpl_sidno sidno= 1; sidno <= max_other_sidno; sidno++)
-      PROPAGATE_REPORTED_ERROR(
-        add_gno_intervals(sidno, Const_interval_iterator(other, sidno),
-                          &lock));
+      add_gno_intervals(sidno, Const_interval_iterator(other, sidno), &lock);
   }
   else
   {
-    /*
-      This code is not being used but we will keep it as it may be
-      useful to optimize gtids by avoiding sharing mappings from
-      sid to sidno. For instance, the IO Thread and the SQL Thread
-      may have different mappings in the future.
-    */
     Sid_map *other_sid_map= other->sid_map;
+    Checkable_rwlock *other_sid_lock= other->sid_lock;
+    if (other_sid_lock != NULL)
+      other_sid_lock->assert_some_wrlock();
     for (rpl_sidno other_sidno= 1; other_sidno <= max_other_sidno;
          other_sidno++)
     {
@@ -687,8 +740,7 @@ enum_return_status Gtid_set::add_gtid_set(const Gtid_set *other)
         if (this_sidno <= 0)
           RETURN_REPORTED_ERROR;
         PROPAGATE_REPORTED_ERROR(ensure_sidno(this_sidno));
-        PROPAGATE_REPORTED_ERROR(add_gno_intervals(this_sidno, other_ivit,
-                                                   &lock));
+        add_gno_intervals(this_sidno, other_ivit, &lock);
       }
     }
   }
@@ -696,7 +748,7 @@ enum_return_status Gtid_set::add_gtid_set(const Gtid_set *other)
 }
 
 
-enum_return_status Gtid_set::remove_gtid_set(const Gtid_set *other)
+void Gtid_set::remove_gtid_set(const Gtid_set *other)
 {
   DBUG_ENTER("Gtid_set::remove_gtid_set(Gtid_set *)");
   if (sid_lock != NULL)
@@ -707,21 +759,15 @@ enum_return_status Gtid_set::remove_gtid_set(const Gtid_set *other)
   {
     rpl_sidno max_sidno= min(max_other_sidno, get_max_sidno());
     for (rpl_sidno sidno= 1; sidno <= max_sidno; sidno++)
-      PROPAGATE_REPORTED_ERROR(
-        remove_gno_intervals(sidno, Const_interval_iterator(other, sidno),
-                             &lock));
+      remove_gno_intervals(sidno, Const_interval_iterator(other, sidno),
+                           &lock);
   }
   else
   {
-    /*
-      This code is not being used but we will keep it as it may be
-      useful to optimize gtids by avoiding sharing mappings from
-      sid to sidno. For instance, the IO Thread and the SQL Thread
-      may have different mappings in the future.
-    */
-    DBUG_ASSERT(0); /*NOTREACHED*/
-#ifdef NON_DISABLED_GTID
     Sid_map *other_sid_map= other->sid_map;
+    Checkable_rwlock *other_sid_lock= other->sid_lock;
+    if (other_sid_lock != NULL)
+      other_sid_lock->assert_some_wrlock();
     for (rpl_sidno other_sidno= 1; other_sidno <= max_other_sidno;
          other_sidno++)
     {
@@ -731,13 +777,11 @@ enum_return_status Gtid_set::remove_gtid_set(const Gtid_set *other)
         const rpl_sid &sid= other_sid_map->sidno_to_sid(other_sidno);
         rpl_sidno this_sidno= sid_map->sid_to_sidno(sid);
         if (this_sidno != 0)
-          PROPAGATE_REPORTED_ERROR(
-            remove_gno_intervals(this_sidno, other_ivit, &lock));
+          remove_gno_intervals(this_sidno, other_ivit, &lock);
       }
     }
-#endif
   }
-  RETURN_OK;
+  DBUG_VOID_RETURN;
 }
 
 
@@ -762,11 +806,35 @@ bool Gtid_set::contains_gtid(rpl_sidno sidno, rpl_gno gno) const
   DBUG_RETURN(false);
 }
 
+rpl_gno Gtid_set::get_last_gno(rpl_sidno sidno) const
+{
+  DBUG_ENTER("Gtid_set::get_last_gno");
+  rpl_gno gno= 0;
+
+  if (sid_lock != NULL)
+    sid_lock->assert_some_lock();
+
+  if (sidno > get_max_sidno())
+    DBUG_RETURN(gno);
+
+  Const_interval_iterator ivit(this, sidno);
+  const Gtid_set::Interval *iv= ivit.get();
+  while(iv != NULL)
+  {
+    gno= iv->end-1;
+    ivit.next();
+    iv= ivit.get();
+  }
+
+  DBUG_RETURN(gno);
+}
+
 int Gtid_set::to_string(char **buf_arg, const Gtid_set::String_format *sf_arg) const
 {
   DBUG_ENTER("Gtid_set::to_string");
   int len= get_string_length(sf_arg);
-  *buf_arg= (char *)my_malloc(len + 1, MYF(MY_WME));
+  *buf_arg= (char *)my_malloc(key_memory_Gtid_set_to_string,
+                              len + 1, MYF(MY_WME));
   if (*buf_arg == NULL)
     DBUG_RETURN(-1);
   to_string(*buf_arg, sf_arg);
@@ -836,22 +904,52 @@ int Gtid_set::to_string(char *buf, const Gtid_set::String_format *sf) const
   memcpy(s, sf->end, sf->end_length);
   s += sf->end_length;
   *s= '\0';
-  DBUG_PRINT("info", ("ret='%s' strlen(s)=%lu s-buf=%lu get_string_length=%d", buf,
-             (ulong) strlen(buf), (ulong) (s - buf), get_string_length(sf)));
+  DBUG_PRINT("info", ("ret='%s' strlen(s)=%zu s-buf=%lu get_string_length=%d", buf,
+             strlen(buf), (ulong) (s - buf), get_string_length(sf)));
   DBUG_ASSERT(s - buf == get_string_length(sf));
   DBUG_RETURN((int)(s - buf));
 }
 
 
+void Gtid_set::get_gtid_intervals(list<Gtid_interval> *gtid_intervals) const
+{
+  DBUG_ENTER("Gtid_set::get_gtid_intervals");
+  DBUG_ASSERT(sid_map != NULL);
+  if (sid_lock != NULL)
+    sid_lock->assert_some_wrlock();
+  rpl_sidno map_max_sidno= sid_map->get_max_sidno();
+  DBUG_ASSERT(get_max_sidno() <= map_max_sidno);
+  for (int sid_i= 0; sid_i < map_max_sidno; sid_i++)
+  {
+    rpl_sidno sidno= sid_map->get_sorted_sidno(sid_i);
+    if (contains_sidno(sidno))
+    {
+      Const_interval_iterator ivit(this, sidno);
+      const Interval *iv= ivit.get();
+      while (iv != NULL)
+      {
+        Gtid_interval gtid_interval;
+        gtid_interval.set(sidno, iv->start, iv->end - 1);
+        gtid_intervals->push_back(gtid_interval);
+        ivit.next();
+        iv= ivit.get();
+      };
+    }
+  }
+
+  DBUG_VOID_RETURN;
+}
+
+
 /**
   Returns the length that the given rpl_sidno (64 bit integer) would
   have, if it was encoded as a string.
 */
-static int get_string_length(rpl_gno gno)
+static size_t get_string_length(rpl_gno gno)
 {
   DBUG_ASSERT(gno >= 1 && gno < MAX_GNO);
   rpl_gno tmp_gno= gno;
-  int len= 0;
+  size_t len= 0;
   do
   {
     tmp_gno /= 10;
@@ -859,7 +957,7 @@ static int get_string_length(rpl_gno gno)
   } while (tmp_gno != 0);
 #ifndef DBUG_OFF
   char buf[22];
-  DBUG_ASSERT(snprintf(buf, 22, "%lld", gno) == len);
+  DBUG_ASSERT(my_snprintf(buf, 22, "%lld", gno) == len);
 #endif
   return len;
 }
@@ -875,7 +973,7 @@ int Gtid_set::get_string_length(const Gtid_set::String_format *sf) const
   if (cached_string_length == -1 || cached_string_format != sf)
   {
     int n_sids= 0, n_intervals= 0, n_long_intervals= 0;
-    int total_interval_length= 0;
+    size_t total_interval_length= 0;
     rpl_sidno max_sidno= get_max_sidno();
     for (rpl_sidno sidno= 1; sidno <= max_sidno; sidno++)
     {
@@ -906,7 +1004,7 @@ int Gtid_set::get_string_length(const Gtid_set::String_format *sf) const
       if (n_sids > 0)
         cached_string_length+=
           total_interval_length +
-          n_sids * (rpl_sid::TEXT_LENGTH + sf->sid_gno_separator_length) +
+          n_sids * (binary_log::Uuid::TEXT_LENGTH + sf->sid_gno_separator_length) +
           (n_sids - 1) * sf->gno_sid_separator_length +
           (n_intervals - n_sids) * sf->gno_gno_separator_length +
           n_long_intervals * sf->gno_start_end_separator_length;
@@ -916,10 +1014,6 @@ int Gtid_set::get_string_length(const Gtid_set::String_format *sf) const
   return cached_string_length;
 }
 
-/*
-  Functions sidno_equals() and equals() are only used by unitests
-*/
-#ifdef NON_DISABLED_UNITTEST_GTID
 bool Gtid_set::sidno_equals(rpl_sidno sidno, const Gtid_set *other,
                             rpl_sidno other_sidno) const
 {
@@ -1011,7 +1105,6 @@ bool Gtid_set::equals(const Gtid_set *other) const
   DBUG_ASSERT(0); // not reached
   DBUG_RETURN(true);
 }
-#endif
 
 
 bool Gtid_set::is_interval_subset(Const_interval_iterator *sub,
@@ -1285,9 +1378,9 @@ Gtid_set::intersection(const Gtid_set *other, Gtid_set *result)
   Gtid_set intersection(sid_map);
   // In set theory, intersection(A, B) == A - (A - B)
   PROPAGATE_REPORTED_ERROR(this_minus_other.add_gtid_set(this));
-  PROPAGATE_REPORTED_ERROR(this_minus_other.remove_gtid_set(other));
+  this_minus_other.remove_gtid_set(other);
   PROPAGATE_REPORTED_ERROR(intersection.add_gtid_set(this));
-  PROPAGATE_REPORTED_ERROR(intersection.remove_gtid_set(&this_minus_other));
+  intersection.remove_gtid_set(&this_minus_other);
   PROPAGATE_REPORTED_ERROR(result->add_gtid_set(&intersection));
   RETURN_OK;
 }
@@ -1320,7 +1413,7 @@ void Gtid_set::encode(uchar *buf) const
       n_sids++;
       // store SID
       sid_map->sidno_to_sid(sidno).copy_to(buf);
-      buf+= rpl_sid::BYTE_LENGTH;
+      buf+= binary_log::Uuid::BYTE_LENGTH;
       // make place for number of intervals
       uint64 n_intervals= 0;
       uchar *n_intervals_p= buf;
@@ -1420,7 +1513,7 @@ enum_return_status Gtid_set::add_gtid_encoding(const uchar *encoded,
       if (current == NULL || start < current->start)
         ivit.init(this, sidno);
       DBUG_PRINT("info", ("adding %d:%lld-%lld", sidno, start, end - 1));
-      PROPAGATE_REPORTED_ERROR(add_gno_interval(&ivit, start, end, &lock));
+      add_gno_interval(&ivit, start, end, &lock);
     }
   }
   DBUG_ASSERT(pos <= length);
