@@ -1,4 +1,4 @@
-/* Copyright (c) 2012, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,7 +14,6 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "my_global.h"    // NO_EMBEDDED_ACCESS_CHECKS
-#include "sql_priv.h"
 #include "sp_instr.h"
 #include "item.h"         // Item_splocal
 #include "opt_trace.h"    // opt_trace_disable_etc
@@ -22,31 +21,37 @@
 #include "sp_head.h"      // sp_head
 #include "sp.h"           // sp_get_item_value
 #include "sp_rcontext.h"  // sp_rcontext
-#include "sql_acl.h"      // SELECT_ACL
+#include "auth_common.h"  // SELECT_ACL
 #include "sql_base.h"     // open_temporary_tables
 #include "sql_parse.h"    // check_table_access
 #include "sql_prepare.h"  // reinit_stmt_before_use
 #include "transaction.h"  // trans_commit_stmt
+#include "prealloced_array.h"
+#include "binlog.h"
+#include "item_cmpfunc.h" // Item_func_eq
 
 #include <algorithm>
+#include <functional>
 
-///////////////////////////////////////////////////////////////////////////
-// Static function implementation.
-///////////////////////////////////////////////////////////////////////////
+#include "trigger.h"                  // Trigger
+#include "table_trigger_dispatcher.h" // Table_trigger_dispatcher
 
 
-static int cmp_splocal_locations(Item_splocal * const *a,
-                                 Item_splocal * const *b)
+class Cmp_splocal_locations :
+  public std::binary_function<const Item_splocal*, const Item_splocal*, bool>
 {
-  return (int)((*a)->pos_in_query - (*b)->pos_in_query);
-}
-
+public:
+  bool operator()(const Item_splocal *a, const Item_splocal *b)
+  {
+    DBUG_ASSERT(a->pos_in_query != b->pos_in_query);
+    return a->pos_in_query < b->pos_in_query;
+  }
+};
 
 /*
   StoredRoutinesBinlogging
   This paragraph applies only to statement-based binlogging. Row-based
-  binlogging does not need anything special like this except for a special
-  case that is mentioned below in section 2.1
+  binlogging does not need anything special like this.
 
   Top-down overview:
 
@@ -73,14 +78,6 @@ static int cmp_splocal_locations(Item_splocal * const *a,
     of SP local variables with NAME_CONST('spvar_name', <spvar-value>) calls.
     This substitution is done in subst_spvars().
 
-  2.1 Miscellaneous case: DDLs (Eg: ALTER EVENT) in StoredProcedure(SP) uses
-      its local variables
-
-  * Irrespective of binlog format, DDLs are always binlogged in statement mode.
-    Hence if there are any DDLs, in stored procedure, that uses SP local
-    variables,  those should be replaced with NAME_CONST('spvar_name', <spvar-value>)
-    even if binlog format is 'row'.
-
   3. FUNCTION calls
 
   In sp_head::execute_function(), we check
@@ -146,10 +143,8 @@ static int cmp_splocal_locations(Item_splocal * const *a,
 */
 static bool subst_spvars(THD *thd, sp_instr *instr, LEX_STRING *query_str)
 {
-  Dynamic_array<Item_splocal*> sp_vars_uses;
-  char *pbuf, *cur, buffer[512];
-  String qbuf(buffer, sizeof(buffer), &my_charset_bin);
-  int prev_pos, res, buf_len;
+  // Stack-local array, does not need instrumentation.
+  Prealloced_array<Item_splocal*, 16> sp_vars_uses(PSI_NOT_INSTRUMENTED);
 
   /* Find all instances of Item_splocal used in this statement */
   for (Item *item= instr->free_list; item; item= item->next)
@@ -158,27 +153,30 @@ static bool subst_spvars(THD *thd, sp_instr *instr, LEX_STRING *query_str)
     {
       Item_splocal *item_spl= (Item_splocal*)item;
       if (item_spl->pos_in_query)
-        sp_vars_uses.append(item_spl);
+        sp_vars_uses.push_back(item_spl);
     }
   }
 
-  if (!sp_vars_uses.elements())
+  if (sp_vars_uses.empty())
     return false;
 
   /* Sort SP var refs by their occurrences in the query */
-  sp_vars_uses.sort(cmp_splocal_locations);
+  std::sort(sp_vars_uses.begin(), sp_vars_uses.end(), Cmp_splocal_locations());
 
   /*
     Construct a statement string where SP local var refs are replaced
     with "NAME_CONST(name, value)"
   */
+  char buffer[512];
+  String qbuf(buffer, sizeof(buffer), &my_charset_bin);
   qbuf.length(0);
-  cur= query_str->str;
-  prev_pos= res= 0;
+  char *cur= query_str->str;
+  int prev_pos= 0;
+  int res= 0;
   thd->query_name_consts= 0;
 
-  for (Item_splocal **splocal= sp_vars_uses.front(); 
-       splocal <= sp_vars_uses.back(); splocal++)
+  for (Item_splocal **splocal= sp_vars_uses.begin(); 
+       splocal != sp_vars_uses.end(); splocal++)
   {
     Item *val;
 
@@ -227,25 +225,11 @@ static bool subst_spvars(THD *thd, sp_instr *instr, LEX_STRING *query_str)
       qbuf.append(cur + prev_pos, query_str->length - prev_pos))
     return true;
 
-  /*
-    Allocate additional space at the end of the new query string for the
-    query_cache_send_result_to_client function.
-
-    The query buffer layout is:
-       buffer :==
-            <statement>   The input statement(s)
-            '\0'          Terminating null char
-            <length>      Length of following current database name (size_t)
-            <db_name>     Name of current database
-            <flags>       Flags struct
-  */
-  buf_len= qbuf.length() + 1 + sizeof(size_t) + thd->db_length + 
-           QUERY_CACHE_FLAGS_SIZE + 1;
-  if ((pbuf= (char *) alloc_root(thd->mem_root, buf_len)))
+  char *pbuf;
+  if ((pbuf= static_cast<char*>(thd->alloc(qbuf.length() + 1))))
   {
     memcpy(pbuf, qbuf.ptr(), qbuf.length());
     pbuf[qbuf.length()]= 0;
-    memcpy(pbuf+qbuf.length()+1, (char *) &thd->db_length, sizeof(size_t));
   }
   else
     return true;
@@ -266,35 +250,6 @@ static bool subst_spvars(THD *thd, sp_instr *instr, LEX_STRING *query_str)
 // sp_lex_instr implementation.
 ///////////////////////////////////////////////////////////////////////////
 
-class SP_instr_error_handler : public Internal_error_handler
-{
-public:
-  SP_instr_error_handler()
-    : cts_table_exists_error(false)
-  {}
-
-  virtual bool handle_condition(THD *thd,
-                                uint sql_errno,
-                                const char*,
-                                Sql_condition::enum_warning_level,
-                                const char*,
-                                Sql_condition **)
-  {
-    /*
-      Check if the "table exists" error or warning reported for the
-      CREATE TABLE ... SELECT statement.
-    */
-    if (thd->lex && thd->lex->sql_command == SQLCOM_CREATE_TABLE &&
-        thd->lex->select_lex.item_list.elements > 0 &&
-        sql_errno == ER_TABLE_EXISTS_ERROR)
-      cts_table_exists_error= true;
-
-    return false;
-  }
-
-  bool cts_table_exists_error;
-};
-
 
 bool sp_lex_instr::reset_lex_and_exec_core(THD *thd,
                                            uint *nextp,
@@ -309,8 +264,8 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd,
   */
 
   unsigned int parent_unsafe_rollback_flags=
-    thd->transaction.stmt.get_unsafe_rollback_flags();
-  thd->transaction.stmt.reset_unsafe_rollback_flags();
+    thd->get_transaction()->get_unsafe_rollback_flags(Transaction_ctx::STMT);
+  thd->get_transaction()->reset_unsafe_rollback_flags(Transaction_ctx::STMT);
 
   /* Check pre-conditions. */
 
@@ -356,8 +311,24 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd,
 
   reinit_stmt_before_use(thd, m_lex);
 
-  SP_instr_error_handler sp_instr_error_handler;
-  thd->push_internal_handler(&sp_instr_error_handler);
+  /*
+    In case a session state exists do not cache the SELECT stmt. If we
+    cache SELECT statment when session state information exists, then
+    the result sets of this SELECT are cached which contains changed
+    session information. Next time when same query is executed when there
+    is no change in session state, then result sets are picked from cache
+    which is wrong as the result sets picked from cache have changed
+    state information.
+    In case of embedded server since session state information is not
+    sent there is no need to turn off cache.
+  */
+
+#ifndef EMBEDDED_LIBRARY
+  if ((thd->client_capabilities & CLIENT_SESSION_TRACK) &&
+      thd->session_tracker.enabled_any() &&
+      thd->session_tracker.changed_any())
+    thd->lex->safe_to_cache_query= 0;
+#endif
 
   /* Open tables if needed. */
 
@@ -386,7 +357,7 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd,
                                UINT_MAX, false));
 
     if (!rc)
-      rc= open_and_lock_tables(thd, m_lex->query_tables, true, 0);
+      rc= open_and_lock_tables(thd, m_lex->query_tables, 0);
 
     if (!rc)
     {
@@ -399,7 +370,7 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd,
       key read.
     */
 
-    m_lex->unit.cleanup();
+    m_lex->unit->cleanup(true);
 
     /* Here we also commit or rollback the current statement. */
 
@@ -432,9 +403,6 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd,
     DBUG_PRINT("info",("exec_core returned: %d", rc));
   }
 
-  // Pop SP_instr_error_handler error handler.
-  thd->pop_internal_handler();
-
   if (m_lex->query_tables_own_last)
   {
     /*
@@ -457,48 +425,14 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd,
   thd->rollback_item_tree_changes();
 
   /*
-    Change state of current arena according to outcome of execution.
-
-    When entering this function, state is STMT_INITIALIZED_FOR_SP if this is
-    the first execution, otherwise it is STMT_EXECUTED.
-
-    When an error occurs during opening tables, no execution takes place and
-    no state change will take place.
-
-    When a re-prepare error is raised, the next execution will re-prepare the
-    statement. To make sure that items are created in the statement mem_root,
-    change state to STMT_INITIALIZED_FOR_SP.
-
-    In other cases, the state should become (or remain) STMT_EXECUTED.
-    See Query_arena->state definition for explanation.
-
-    Some special handling of CREATE TABLE .... SELECT in an SP is required. The
-    state is set to STMT_INITIALIZED_FOR_SP even in case of "table exists"
-    error situation.
-
-    Why is this necessary? A useful pointer would be to note how
-    PREPARE/EXECUTE uses functions like select_like_stmt_test to implement
-    CREATE TABLE .... SELECT. The SELECT part of the DDL is resolved first.
-    Then there is an attempt to create the table. So in the execution phase,
-    if "table exists" error occurs or flush table preceeds the execute, the
-    item tree of the select is re-created and followed by an attempt to create
-    the table.
-
-    But SP uses mysql_execute_command (which is used by the conventional
-    execute) after doing a parse. This creates a problem for SP since it
-    tries to preserve the item tree from the previous execution.
+    Update the state of the active arena if no errors on
+    open_tables stage.
   */
 
-  bool reprepare_error=
-    rc && thd->is_error() &&
-    thd->get_stmt_da()->sql_errno() == ER_NEED_REPREPARE;
-
-  if (reprepare_error || sp_instr_error_handler.cts_table_exists_error)
-    thd->stmt_arena->state= Query_arena::STMT_INITIALIZED_FOR_SP;
-  else if (!rc || !thd->is_error() ||
-           (thd->get_stmt_da()->sql_errno() != ER_CANT_REOPEN_TABLE &&
-            thd->get_stmt_da()->sql_errno() != ER_NO_SUCH_TABLE &&
-            thd->get_stmt_da()->sql_errno() != ER_UPDATE_TABLE_USED))
+  if (!rc || !thd->is_error() ||
+      (thd->get_stmt_da()->mysql_errno() != ER_CANT_REOPEN_TABLE &&
+       thd->get_stmt_da()->mysql_errno() != ER_NO_SUCH_TABLE &&
+       thd->get_stmt_da()->mysql_errno() != ER_UPDATE_TABLE_USED))
     thd->stmt_arena->state= Query_arena::STMT_EXECUTED;
 
   /*
@@ -506,7 +440,9 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd,
     what is needed from the substatement gained
   */
 
-  thd->transaction.stmt.add_unsafe_rollback_flags(parent_unsafe_rollback_flags);
+  thd->get_transaction()->add_unsafe_rollback_flags(
+    Transaction_ctx::STMT,
+    parent_unsafe_rollback_flags);
 
   /* Restore original lex. */
 
@@ -553,7 +489,8 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp)
 
   // Cleanup and re-init the lex mem_root for re-parse.
   free_root(&m_lex_mem_root, MYF(0));
-  init_sql_alloc(&m_lex_mem_root, MEM_ROOT_BLOCK_SIZE, MEM_ROOT_PREALLOC);
+  init_sql_alloc(PSI_NOT_INSTRUMENTED, &m_lex_mem_root,
+                 MEM_ROOT_BLOCK_SIZE, MEM_ROOT_PREALLOC);
 
   /*
     Switch mem-roots. We store the new LEX and its Items in the
@@ -562,7 +499,7 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp)
     initiated. Also set the statement query arena to the lex mem_root.
   */
   MEM_ROOT *execution_mem_root= thd->mem_root;
-  Query_arena parse_arena(&m_lex_mem_root, thd->stmt_arena->state);
+  Query_arena parse_arena(&m_lex_mem_root, STMT_INITIALIZED_FOR_SP);
 
   thd->mem_root= &m_lex_mem_root;
   thd->stmt_arena->set_query_arena(&parse_arena);
@@ -618,19 +555,21 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp)
         execution.
       */
 
-      Table_triggers_list *ttl= sp->m_trg_list;
-      int event= sp->m_trg_chistics.event;
-      int action_time= sp->m_trg_chistics.action_time;
-      GRANT_INFO *grant_table= &ttl->subject_table_grants[event][action_time];
+      Trigger *t= sp->m_trg_list->find_trigger(thd->lex->sphead->m_name);
 
-      for (Item_trigger_field *trg_field=
-             sp->m_cur_instr_trig_field_items.first;
-           trg_field;
-           trg_field= trg_field->next_trg_field)
+      DBUG_ASSERT(t);
+
+      if (!t)
+        return NULL; // Don't take chances in production.
+
+      for (Item_trigger_field *trg_fld= sp->m_cur_instr_trig_field_items.first;
+           trg_fld;
+           trg_fld= trg_fld->next_trg_field)
       {
-        trg_field->setup_field(thd, ttl->trigger_table, grant_table);
+        trg_fld->setup_field(thd, sp->m_trg_list->get_trigger_field_support(),
+                             t->get_subject_table_grant());
       }
-     
+
       /**
         Move Item_trigger_field's list to instruction's Item_trigger_field
         list.
@@ -740,7 +679,7 @@ bool sp_lex_instr::validate_lex_and_execute_core(THD *thd,
     if (stmt_reprepare_observer &&
         !thd->is_fatal_error &&
         !thd->killed &&
-        thd->get_stmt_da()->sql_errno() == ER_NEED_REPREPARE &&
+        thd->get_stmt_da()->mysql_errno() == ER_NEED_REPREPARE &&
         reprepare_attempt++ < 3)
     {
       DBUG_ASSERT(stmt_reprepare_observer->is_invalidated());
@@ -819,6 +758,10 @@ void sp_lex_instr::get_query(String *sql_query) const
 // sp_instr_stmt implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_stmt::psi_info=                                    
+{ 0, "stmt", 0};
+#endif
 
 bool sp_instr_stmt::execute(THD *thd, uint *nextp)
 {
@@ -827,7 +770,9 @@ bool sp_instr_stmt::execute(THD *thd, uint *nextp)
 
   DBUG_PRINT("info", ("query: '%.*s'", (int) m_query.length, m_query.str));
 
-  const CSET_STRING query_backup= thd->query_string;
+  MYSQL_SET_STATEMENT_TEXT(thd->m_statement_psi, m_query.str, m_query.length);
+
+  const LEX_CSTRING query_backup= thd->query();
 
 #if defined(ENABLED_PROFILING)
   /* This SP-instr is profilable and will be captured. */
@@ -847,13 +792,7 @@ bool sp_instr_stmt::execute(THD *thd, uint *nextp)
 
     - general log is off
 
-    - binary logging is off
-
-    - if the query generates row events in binlog row format mode
-    (DDLs are always written in statement format irrespective of binlog_format
-    and they can have SP variables in it. For example, 'ALTER EVENT' is allowed
-    inside a procedure and can contain SP variables in it. Those too need to be
-    substituted with NAME_CONST(...))
+    - binary logging is off, or not in statement mode
 
     We don't have to substitute on behalf of the query cache as
     queries with SP vars are not cached, anyway.
@@ -870,11 +809,10 @@ bool sp_instr_stmt::execute(THD *thd, uint *nextp)
     substitution immediately before writing to the log.
   */
 
-  need_subst= !((thd->variables.option_bits & OPTION_LOG_OFF) &&
+  need_subst= ((thd->variables.option_bits & OPTION_LOG_OFF) &&
                (!(thd->variables.option_bits & OPTION_BIN_LOG) ||
                 !mysql_bin_log.is_open() ||
-                (thd->is_current_stmt_binlog_format_row() &&
-                 sqlcom_can_generate_row_events(m_lex->sql_command))));
+                thd->is_current_stmt_binlog_format_row())) ? FALSE : TRUE;
 
   /*
     If we need to do a substitution but can't (OOM), give up.
@@ -888,10 +826,10 @@ bool sp_instr_stmt::execute(THD *thd, uint *nextp)
     queries with SP vars can't be cached)
   */
   if (unlikely((thd->variables.option_bits & OPTION_LOG_OFF)==0))
-    general_log_write(thd, COM_QUERY, thd->query(), thd->query_length());
+    query_logger.general_log_write(thd, COM_QUERY, thd->query().str,
+                                   thd->query().length);
 
-  if (query_cache_send_result_to_client(thd, thd->query(),
-                                        thd->query_length()) <= 0)
+  if (query_cache.send_result_to_client(thd, thd->query()) <= 0)
   {
     rc= validate_lex_and_execute_core(thd, nextp, false);
 
@@ -903,7 +841,7 @@ bool sp_instr_stmt::execute(THD *thd, uint *nextp)
       thd->protocol->end_statement();
     }
 
-    query_cache_end_of_result(thd);
+    query_cache.end_of_result(thd);
 
     if (!rc && unlikely(log_slow_applicable(thd)))
     {
@@ -934,9 +872,6 @@ bool sp_instr_stmt::execute(THD *thd, uint *nextp)
   thd->set_query(query_backup);
   thd->query_name_consts= 0;
 
-  if (!thd->is_error())
-    thd->get_stmt_da()->reset_diagnostics_area();
-
   return rc || thd->is_error();
 }
 
@@ -953,12 +888,12 @@ void sp_instr_stmt::print(String *str)
     Print the query string (but not too much of it), just to indicate which
     statement it is.
   */
-  uint len= m_query.length;
+  size_t len= m_query.length;
   if (len > SP_STMT_PRINT_MAXLEN)
     len= SP_STMT_PRINT_MAXLEN-3;
 
   /* Copy the query string and replace '\n' with ' ' in the process */
-  for (uint i= 0 ; i < len ; i++)
+  for (size_t i= 0 ; i < len ; i++)
   {
     char c= m_query.str[i];
     if (c == '\n')
@@ -967,24 +902,23 @@ void sp_instr_stmt::print(String *str)
   }
   if (m_query.length > SP_STMT_PRINT_MAXLEN)
     str->qs_append(STRING_WITH_LEN("...")); /* Indicate truncated string */
-  str->qs_append('"');
+  str->qs_append(STRING_WITH_LEN("\""));
 }
 
 
 bool sp_instr_stmt::exec_core(THD *thd, uint *nextp)
 {
-  MYSQL_QUERY_EXEC_START(thd->query(),
-                         thd->thread_id,
-                         (char *) (thd->db ? thd->db : ""),
-                         &thd->security_ctx->priv_user[0],
-                         (char *)thd->security_ctx->host_or_ip,
+  MYSQL_QUERY_EXEC_START(const_cast<char*>(thd->query().str),
+                         thd->thread_id(),
+                         (char *) (thd->db().str ? thd->db().str : ""),
+                         (char *) thd->security_context()->priv_user().str,
+                         (char *) thd->security_context()->host_or_ip().str,
                          3);
 
   thd->lex->set_sp_current_parsing_ctx(get_parsing_ctx());
   thd->lex->sphead= thd->sp_runtime_ctx->sp;
 
   PSI_statement_locker *statement_psi_saved= thd->m_statement_psi;
-  thd->m_statement_psi= NULL;
 
   bool rc= mysql_execute_command(thd);
 
@@ -1004,6 +938,10 @@ bool sp_instr_stmt::exec_core(THD *thd, uint *nextp)
 // sp_instr_set implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_set::psi_info=                                     
+{ 0, "set", 0};
+#endif
 
 bool sp_instr_set::exec_core(THD *thd, uint *nextp)
 {
@@ -1027,7 +965,7 @@ bool sp_instr_set::exec_core(THD *thd, uint *nextp)
 void sp_instr_set::print(String *str)
 {
   /* set name@offset ... */
-  int rsrv = SP_INSTR_UINT_MAXLEN+6;
+  size_t rsrv = SP_INSTR_UINT_MAXLEN+6;
   sp_variable *var = m_parsing_ctx->find_variable(m_offset);
 
   /* 'var' should always be non-null, but just in case... */
@@ -1043,7 +981,7 @@ void sp_instr_set::print(String *str)
   }
   str->qs_append(m_offset);
   str->qs_append(' ');
-  m_value_item->print(str, QT_ORDINARY);
+  m_value_item->print(str, QT_TO_ARGUMENT_CHARSET);
 }
 
 
@@ -1051,12 +989,29 @@ void sp_instr_set::print(String *str)
 // sp_instr_set_trigger_field implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_set_trigger_field::psi_info=                       
+{ 0, "set_trigger_field", 0};
+#endif
 
 bool sp_instr_set_trigger_field::exec_core(THD *thd, uint *nextp)
 {
   *nextp= get_ip() + 1;
   thd->count_cuted_fields= CHECK_FIELD_ERROR_FOR_NULL;
-  return m_trigger_field->set_value(thd, &m_value_item);
+  Strict_error_handler strict_handler(Strict_error_handler::
+                                      ENABLE_SET_SELECT_STRICT_ERROR_HANDLER);
+  /*
+    Before Triggers are executed after the 'data' is assigned
+    to the Field objects. If triggers wants to SET invalid value
+    to the Field objects (NEW.<variable_name>= <Invalid value>),
+    it should not be allowed.
+  */
+  if (thd->is_strict_mode() && !thd->lex->is_ignore())
+    thd->push_internal_handler(&strict_handler);
+  bool error= m_trigger_field->set_value(thd, &m_value_item);
+  if (thd->is_strict_mode() && !thd->lex->is_ignore())
+    thd->pop_internal_handler();
+  return error;
 }
 
 
@@ -1065,21 +1020,21 @@ void sp_instr_set_trigger_field::print(String *str)
   str->append(STRING_WITH_LEN("set_trigger_field "));
   m_trigger_field->print(str, QT_ORDINARY);
   str->append(STRING_WITH_LEN(":="));
-  m_value_item->print(str, QT_ORDINARY);
+  m_value_item->print(str, QT_TO_ARGUMENT_CHARSET);
 }
 
 
 bool sp_instr_set_trigger_field::on_after_expr_parsing(THD *thd)
 {
-  DBUG_ASSERT(thd->lex->select_lex.item_list.elements == 1);
+  DBUG_ASSERT(thd->lex->select_lex->item_list.elements == 1);
 
-  m_value_item= thd->lex->select_lex.item_list.head();
+  m_value_item= thd->lex->select_lex->item_list.head();
 
   DBUG_ASSERT(!m_trigger_field);
 
   m_trigger_field=
     new (thd->mem_root) Item_trigger_field(thd->lex->current_context(),
-                                           Item_trigger_field::NEW_ROW,
+                                           TRG_NEW_ROW,
                                            m_trigger_field_name.str,
                                            UPDATE_ACL,
                                            false);
@@ -1108,6 +1063,10 @@ void sp_instr_set_trigger_field::cleanup_before_parsing(THD *thd)
 // sp_instr_jump implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_jump::psi_info=                                    
+{ 0, "jump", 0};
+#endif
 
 void sp_instr_jump::print(String *str)
 {
@@ -1163,6 +1122,10 @@ void sp_instr_jump::opt_move(uint dst, List<sp_branch_instr> *bp)
 // sp_instr_jump_if_not class implementation
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_jump_if_not::psi_info=                             
+{ 0, "jump_if_not", 0};
+#endif
 
 bool sp_instr_jump_if_not::exec_core(THD *thd, uint *nextp)
 {
@@ -1255,6 +1218,10 @@ void sp_lex_branch_instr::opt_move(uint dst, List<sp_branch_instr> *bp)
 // sp_instr_jump_case_when implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_jump_case_when::psi_info=                          
+{ 0, "jump_case_when", 0};
+#endif
 
 bool sp_instr_jump_case_when::exec_core(THD *thd, uint *nextp)
 {
@@ -1312,9 +1279,9 @@ bool sp_instr_jump_case_when::build_expr_items(THD *thd)
 
   if (!m_expr_item)
   {
-    DBUG_ASSERT(thd->lex->select_lex.item_list.elements == 1);
+    DBUG_ASSERT(thd->lex->select_lex->item_list.elements == 1);
 
-    m_expr_item= thd->lex->select_lex.item_list.head();
+    m_expr_item= thd->lex->select_lex->item_list.head();
   }
 
   // Setup main expression item (m_expr_item).
@@ -1332,17 +1299,13 @@ bool sp_instr_jump_case_when::build_expr_items(THD *thd)
 // sp_instr_freturn implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_freturn::psi_info=                                 
+{ 0, "freturn", 0};
+#endif
 
 bool sp_instr_freturn::exec_core(THD *thd, uint *nextp)
 {
-  /*
-    RETURN is a "procedure statement" (in terms of the SQL standard).
-    That means, Diagnostics Area should be clean before its execution.
-  */
-
-  Diagnostics_area *da= thd->get_stmt_da();
-  da->clear_warning_info(da->warning_info_id());
-
   /*
     Change <next instruction pointer>, so that this will be the last
     instruction in the stored function.
@@ -1378,6 +1341,10 @@ void sp_instr_freturn::print(String *str)
 // sp_instr_hpush_jump implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_hpush_jump::psi_info=                              
+{ 0, "hpush_jump", 0};
+#endif
 
 bool sp_instr_hpush_jump::execute(THD *thd, uint *nextp)
 {
@@ -1398,17 +1365,7 @@ void sp_instr_hpush_jump::print(String *str)
   str->qs_append(' ');
   str->qs_append(m_frame);
 
-  switch (m_handler->type) {
-  case sp_handler::EXIT:
-    str->qs_append(STRING_WITH_LEN(" EXIT"));
-    break;
-  case sp_handler::CONTINUE:
-    str->qs_append(STRING_WITH_LEN(" CONTINUE"));
-    break;
-  default:
-    // The handler type must be either CONTINUE or EXIT.
-    DBUG_ASSERT(0);
-  }
+  m_handler->print(str);
 }
 
 
@@ -1450,6 +1407,10 @@ uint sp_instr_hpush_jump::opt_mark(sp_head *sp, List<sp_instr> *leads)
 // sp_instr_hpop implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_hpop::psi_info=                                    
+{ 0, "hpop", 0};
+#endif
 
 bool sp_instr_hpop::execute(THD *thd, uint *nextp)
 {
@@ -1463,16 +1424,13 @@ bool sp_instr_hpop::execute(THD *thd, uint *nextp)
 // sp_instr_hreturn implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_hreturn::psi_info=                                 
+{ 0, "hreturn", 0};
+#endif
 
 bool sp_instr_hreturn::execute(THD *thd, uint *nextp)
 {
-  /*
-    Remove the SQL conditions that were present in DA when the
-    handler was activated.
-  */
-
-  thd->get_stmt_da()->remove_marked_sql_conditions();
-
   /*
     Obtain next instruction pointer (m_dest is set for EXIT handlers, retrieve
     the instruction pointer from runtime context for CONTINUE handlers).
@@ -1488,7 +1446,7 @@ bool sp_instr_hreturn::execute(THD *thd, uint *nextp)
   */
 
   sp_instr *next_instr= rctx->sp->get_instr(*nextp);
-  rctx->exit_handler(next_instr->get_parsing_ctx());
+  rctx->exit_handler(thd, next_instr->get_parsing_ctx());
 
   return false;
 }
@@ -1538,6 +1496,10 @@ uint sp_instr_hreturn::opt_mark(sp_head *sp, List<sp_instr> *leads)
 // sp_instr_cpush implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_cpush::psi_info=                                   
+{ 0, "cpush", 0};
+#endif
 
 bool sp_instr_cpush::execute(THD *thd, uint *nextp)
 {
@@ -1564,7 +1526,7 @@ void sp_instr_cpush::print(String *str)
 {
   const LEX_STRING *cursor_name= m_parsing_ctx->find_cursor(m_cursor_idx);
 
-  uint rsrv= SP_INSTR_UINT_MAXLEN + 7 + m_cursor_query.length + 1;
+  size_t rsrv= SP_INSTR_UINT_MAXLEN + 7 + m_cursor_query.length + 1;
 
   if (cursor_name)
     rsrv+= cursor_name->length;
@@ -1587,6 +1549,10 @@ void sp_instr_cpush::print(String *str)
 // sp_instr_cpop implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_cpop::psi_info=                                    
+{ 0, "cpop", 0};
+#endif
 
 bool sp_instr_cpop::execute(THD *thd, uint *nextp)
 {
@@ -1611,9 +1577,16 @@ void sp_instr_cpop::print(String *str)
 // sp_instr_copen implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_copen::psi_info=                                   
+{ 0, "copen", 0};
+#endif
 
 bool sp_instr_copen::execute(THD *thd, uint *nextp)
 {
+  // Manipulating a CURSOR with an expression should clear DA.
+  clear_da(thd);
+
   *nextp= get_ip() + 1;
 
   // Get the cursor pointer.
@@ -1659,7 +1632,7 @@ void sp_instr_copen::print(String *str)
   const LEX_STRING *cursor_name= m_parsing_ctx->find_cursor(m_cursor_idx);
 
   /* copen name@offset */
-  uint rsrv= SP_INSTR_UINT_MAXLEN+7;
+  size_t rsrv= SP_INSTR_UINT_MAXLEN+7;
 
   if (cursor_name)
     rsrv+= cursor_name->length;
@@ -1679,9 +1652,16 @@ void sp_instr_copen::print(String *str)
 // sp_instr_cclose implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_cclose::psi_info=                                  
+{ 0, "cclose", 0};
+#endif
 
 bool sp_instr_cclose::execute(THD *thd, uint *nextp)
 {
+  // Manipulating a CURSOR with an expression should clear DA.
+  clear_da(thd);
+
   *nextp= get_ip() + 1;
 
   sp_cursor *c= thd->sp_runtime_ctx->get_cursor(m_cursor_idx);
@@ -1695,7 +1675,7 @@ void sp_instr_cclose::print(String *str)
   const LEX_STRING *cursor_name= m_parsing_ctx->find_cursor(m_cursor_idx);
 
   /* cclose name@offset */
-  uint rsrv= SP_INSTR_UINT_MAXLEN+8;
+  size_t rsrv= SP_INSTR_UINT_MAXLEN+8;
 
   if (cursor_name)
     rsrv+= cursor_name->length;
@@ -1715,9 +1695,16 @@ void sp_instr_cclose::print(String *str)
 // sp_instr_cfetch implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_cfetch::psi_info=                                  
+{ 0, "cfetch", 0};
+#endif
 
 bool sp_instr_cfetch::execute(THD *thd, uint *nextp)
 {
+  // Manipulating a CURSOR with an expression should clear DA.
+  clear_da(thd);
+
   *nextp= get_ip() + 1;
 
   sp_cursor *c= thd->sp_runtime_ctx->get_cursor(m_cursor_idx);
@@ -1733,7 +1720,7 @@ void sp_instr_cfetch::print(String *str)
   const LEX_STRING *cursor_name= m_parsing_ctx->find_cursor(m_cursor_idx);
 
   /* cfetch name@offset vars... */
-  uint rsrv= SP_INSTR_UINT_MAXLEN+8;
+  size_t rsrv= SP_INSTR_UINT_MAXLEN+8;
 
   if (cursor_name)
     rsrv+= cursor_name->length;
@@ -1762,6 +1749,10 @@ void sp_instr_cfetch::print(String *str)
 // sp_instr_error implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_error::psi_info=                                   
+{ 0, "error", 0};
+#endif
 
 void sp_instr_error::print(String *str)
 {
@@ -1777,6 +1768,10 @@ void sp_instr_error::print(String *str)
 // sp_instr_set_case_expr implementation.
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_PSI_INTERFACE
+PSI_statement_info sp_instr_set_case_expr::psi_info=                           
+{ 0, "set_case_expr", 0};
+#endif
 
 bool sp_instr_set_case_expr::exec_core(THD *thd, uint *nextp)
 {
