@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,10 +16,7 @@
 #ifndef SQL_BASE_INCLUDED
 #define SQL_BASE_INCLUDED
 
-#include "unireg.h"                    // REQUIRED: for other includes
-#include "sql_trigger.h"                        /* trg_event_type */
 #include "sql_class.h"                          /* enum_mark_columns */
-#include "mysqld.h"                             /* key_map */
 
 class Item_ident;
 struct Name_resolution_context;
@@ -30,11 +27,64 @@ struct TABLE_LIST;
 class THD;
 struct handlerton;
 struct TABLE;
+class Table_trigger_dispatcher;
 
 typedef class st_select_lex SELECT_LEX;
 
 typedef struct st_lock_param_type ALTER_PARTITION_PARAM_TYPE;
 
+#define TEMP_PREFIX	"MY"
+
+	/* Defines for use with openfrm, openprt and openfrd */
+
+#define READ_ALL		1	/* openfrm: Read all parameters */
+#define EXTRA_RECORD		8	/* Reservera plats f|r extra record */
+#define DELAYED_OPEN	        4096    /* Open table later */
+#define OPEN_VIEW		8196    /* Allow open on view */
+#define OPEN_VIEW_NO_PARSE     16384    /* Open frm only if it's a view,
+                                           but do not parse view itself */
+/**
+  This flag is used in function get_all_tables() which fills
+  I_S tables with data which are retrieved from frm files and storage engine
+  The flag means that we need to open FRM file only to get necessary data.
+*/
+#define OPEN_FRM_FILE_ONLY     32768
+/**
+  This flag is used in function get_all_tables() which fills
+  I_S tables with data which are retrieved from frm files and storage engine
+  The flag means that we need to process tables only to get necessary data.
+  Views are not processed.
+*/
+#define OPEN_TABLE_ONLY        OPEN_FRM_FILE_ONLY*2
+/**
+  This flag is used in function get_all_tables() which fills
+  I_S tables with data which are retrieved from frm files and storage engine
+  The flag means that we need to process views only to get necessary data.
+  Tables are not processed.
+*/
+#define OPEN_VIEW_ONLY         OPEN_TABLE_ONLY*2
+/**
+  This flag is used in function get_all_tables() which fills
+  I_S tables with data which are retrieved from frm files and storage engine.
+  The flag means that we need to open a view.
+*/
+#define OPEN_VIEW_FULL         OPEN_VIEW_ONLY*2
+/**
+  This flag is used in function get_all_tables() which fills
+  I_S tables with data which are retrieved from frm files and storage engine.
+  The flag means that I_S table uses optimization algorithm.
+*/
+#define OPTIMIZE_I_S_TABLE     OPEN_VIEW_FULL*2
+/**
+  The flag means that we need to process trigger files only.
+*/
+#define OPEN_TRIGGER_ONLY      OPTIMIZE_I_S_TABLE*2
+/**
+  This flag is used to instruct tdc_open_view() to check metadata version.
+*/
+#define CHECK_METADATA_VERSION OPEN_TRIGGER_ONLY*2
+
+
 /*
   This enumeration type is used only by the function find_item_in_list
   to return the info on how an item has been resolved against a list
@@ -78,13 +128,14 @@ void table_def_free(void);
 void table_def_start_shutdown(void);
 void assign_new_table_id(TABLE_SHARE *share);
 uint cached_table_definitions(void);
-uint get_table_def_key(const TABLE_LIST *table_list, const char **key);
+size_t get_table_def_key(const TABLE_LIST *table_list, const char **key);
 TABLE_SHARE *get_table_share(THD *thd, TABLE_LIST *table_list,
-                             const char *key, uint key_length,
+                             const char *key, size_t key_length,
                              uint db_flags, int *error,
                              my_hash_value_type hash_value);
 void release_table_share(TABLE_SHARE *share);
-TABLE_SHARE *get_cached_table_share(const char *db, const char *table_name);
+TABLE_SHARE *get_cached_table_share(THD *thd, const char *db,
+                                    const char *table_name);
 
 TABLE *open_ltable(THD *thd, TABLE_LIST *table_list, thr_lock_type update,
                    uint lock_flags);
@@ -122,7 +173,7 @@ TABLE *open_ltable(THD *thd, TABLE_LIST *table_list, thr_lock_type update,
 #define MYSQL_LOCK_IGNORE_TIMEOUT               0x0800
 /**
   When acquiring "strong" (SNW, SNRW, X) metadata locks on tables to
-  be open do not acquire global and schema-scope IX locks.
+  be open do not acquire global, tablespace-scope and schema-scope IX locks.
 */
 #define MYSQL_OPEN_SKIP_SCOPED_MDL_LOCK         0x1000
 /**
@@ -170,63 +221,55 @@ TABLE_LIST *find_table_in_list(TABLE_LIST *table,
 TABLE *find_temporary_table(THD *thd, const char *db, const char *table_name);
 TABLE *find_temporary_table(THD *thd, const TABLE_LIST *tl);
 TABLE *find_temporary_table(THD *thd, const char *table_key,
-                            uint table_key_length);
+                            size_t table_key_length);
 void close_thread_tables(THD *thd);
 bool fill_record_n_invoke_before_triggers(THD *thd, List<Item> &fields,
                                           List<Item> &values,
-                                          bool ignore_errors,
-                                          Table_triggers_list *triggers,
-                                          enum trg_event_type event);
+                                          TABLE *table,
+                                          enum enum_trigger_event_type event,
+                                          int num_fields);
 bool fill_record_n_invoke_before_triggers(THD *thd, Field **field,
                                           List<Item> &values,
-                                          bool ignore_errors,
-                                          Table_triggers_list *triggers,
-                                          enum trg_event_type event);
+                                          TABLE *table,
+                                          enum enum_trigger_event_type event,
+                                          int num_fields);
 bool insert_fields(THD *thd, Name_resolution_context *context,
 		   const char *db_name, const char *table_name,
                    List_iterator<Item> *it, bool any_privileges);
-int setup_wild(THD *thd, TABLE_LIST *tables, List<Item> &fields,
-	       List<Item> *sum_func_list, uint wild_num);
 bool setup_fields(THD *thd, Ref_ptr_array ref_pointer_array,
-                  List<Item> &item, enum_mark_columns mark_used_columns,
+                  List<Item> &item, ulong privilege,
                   List<Item> *sum_func_list, bool allow_sum_func);
 bool fill_record(THD * thd, List<Item> &fields, List<Item> &values,
-                 bool ignore_errors, MY_BITMAP *bitmap);
+                 MY_BITMAP *bitmap, MY_BITMAP *insert_into_fields_bitmap);
 bool fill_record(THD *thd, Field **field, List<Item> &values,
-                 bool ignore_errors, MY_BITMAP *bitmap);
+                 MY_BITMAP *bitmap, MY_BITMAP *insert_into_fields_bitmap);
+
+bool check_record(THD *thd, Field **ptr);
 
 Field *
 find_field_in_tables(THD *thd, Item_ident *item,
                      TABLE_LIST *first_table, TABLE_LIST *last_table,
                      Item **ref, find_item_error_report_type report_error,
-                     bool check_privileges, bool register_tree_change);
+                     ulong want_privilege, bool register_tree_change);
 Field *
 find_field_in_table_ref(THD *thd, TABLE_LIST *table_list,
-                        const char *name, uint length,
+                        const char *name, size_t length,
                         const char *item_name, const char *db_name,
                         const char *table_name, Item **ref,
-                        bool check_privileges, bool allow_rowid,
+                        ulong want_privilege, bool allow_rowid,
                         uint *cached_field_index_ptr,
                         bool register_tree_change, TABLE_LIST **actual_table);
 Field *
-find_field_in_table(THD *thd, TABLE *table, const char *name, uint length,
+find_field_in_table(THD *thd, TABLE *table, const char *name, size_t length,
                     bool allow_rowid, uint *cached_field_index_ptr);
 Field *
 find_field_in_table_sef(TABLE *table, const char *name);
 Item ** find_item_in_list(Item *item, List<Item> &items, uint *counter,
                           find_item_error_report_type report_error,
                           enum_resolution_type *resolution);
-bool setup_tables(THD *thd, Name_resolution_context *context,
-                  List<TABLE_LIST> *from_clause, TABLE_LIST *tables,
-                  TABLE_LIST **leaves, bool select_insert);
-bool setup_tables_and_check_access(THD *thd,
-                                   Name_resolution_context *context,
-                                   List<TABLE_LIST> *from_clause,
-                                   TABLE_LIST *tables,
-                                   TABLE_LIST **leaves,
-                                   bool select_insert,
-                                   ulong want_access_first,
-                                   ulong want_access);
+bool setup_natural_join_row_types(THD *thd,
+                                  List<TABLE_LIST> *from_clause,
+                                  Name_resolution_context *context);
 bool wait_while_table_is_used(THD *thd, TABLE *table,
                               enum ha_extra_function function);
 
@@ -235,31 +278,29 @@ void drop_open_table(THD *thd, TABLE *table, const char *db_name,
 void update_non_unique_table_error(TABLE_LIST *update,
                                    const char *operation,
                                    TABLE_LIST *duplicate);
-int setup_conds(THD *thd, TABLE_LIST *tables, TABLE_LIST *leaves,
-		Item **conds);
 int setup_ftfuncs(SELECT_LEX* select);
-int init_ftfuncs(THD *thd, SELECT_LEX* select, bool no_order);
+int init_ftfuncs(THD *thd, SELECT_LEX* select);
+int run_before_dml_hook(THD *thd);
 bool lock_table_names(THD *thd, TABLE_LIST *table_list,
                       TABLE_LIST *table_list_end, ulong lock_wait_timeout,
                       uint flags);
 bool open_tables(THD *thd, TABLE_LIST **tables, uint *counter, uint flags,
                  Prelocking_strategy *prelocking_strategy);
-/* open_and_lock_tables with optional derived handling */
-bool open_and_lock_tables(THD *thd, TABLE_LIST *tables,
-                          bool derived, uint flags,
+/* open_and_lock_tables */
+bool open_and_lock_tables(THD *thd, TABLE_LIST *tables, uint flags,
                           Prelocking_strategy *prelocking_strategy);
-/* simple open_and_lock_tables without derived handling for single table */
+/* simple open_and_lock_tables for single table */
 TABLE *open_n_lock_single_table(THD *thd, TABLE_LIST *table_l,
                                 thr_lock_type lock_type, uint flags,
                                 Prelocking_strategy *prelocking_strategy);
-bool open_normal_and_derived_tables(THD *thd, TABLE_LIST *tables, uint flags);
+bool open_tables_for_query(THD *thd, TABLE_LIST *tables, uint flags);
 bool lock_tables(THD *thd, TABLE_LIST *tables, uint counter, uint flags);
 void free_io_cache(TABLE *entry);
 void intern_close_table(TABLE *entry);
 void close_thread_table(THD *thd, TABLE **table_ptr);
 bool close_temporary_tables(THD *thd);
-TABLE_LIST *unique_table(THD *thd, TABLE_LIST *table, TABLE_LIST *table_list,
-                         bool check_alias);
+TABLE_LIST *unique_table(THD *thd, const TABLE_LIST *table,
+                         TABLE_LIST *table_list, bool check_alias);
 int drop_temporary_table(THD *thd, TABLE_LIST *table_list, bool *is_trans);
 void close_temporary_table(THD *thd, TABLE *table, bool free_share,
                            bool delete_table);
@@ -271,9 +312,11 @@ bool open_temporary_table(THD *thd, TABLE_LIST *tl);
 bool is_equal(const LEX_STRING *a, const LEX_STRING *b);
 
 /* Functions to work with system tables. */
-bool open_system_tables_for_read(THD *thd, TABLE_LIST *table_list,
+bool open_nontrans_system_tables_for_read(THD *thd, TABLE_LIST *table_list,
                                  Open_tables_backup *backup);
-void close_system_tables(THD *thd, Open_tables_backup *backup);
+bool open_trans_system_tables_for_read(THD *thd, TABLE_LIST *table_list);
+void close_nontrans_system_tables(THD *thd, Open_tables_backup *backup);
+void close_trans_system_tables(THD *thd);
 void close_mysql_tables(THD *thd);
 TABLE *open_system_table_for_update(THD *thd, TABLE_LIST *one_table);
 TABLE *open_log_table(THD *thd, TABLE_LIST *one_table, Open_tables_backup *backup);
@@ -285,7 +328,6 @@ void close_performance_schema_table(THD *thd, Open_tables_state *backup);
 
 bool close_cached_tables(THD *thd, TABLE_LIST *tables,
                          bool wait_for_refresh, ulong timeout);
-bool close_cached_connection_tables(THD *thd, LEX_STRING *connect_string);
 void close_all_tables_for_name(THD *thd, TABLE_SHARE *share,
                                bool remove_from_locked_tables,
                                TABLE *skip_table);
@@ -294,7 +336,7 @@ void tdc_remove_table(THD *thd, enum_tdc_remove_table_type remove_type,
                       const char *db, const char *table_name,
                       bool has_lock);
 bool tdc_open_view(THD *thd, TABLE_LIST *table_list, const char *alias,
-                   const char *cache_key, uint cache_key_length, uint flags);
+                   const char *cache_key, size_t cache_key_length, uint flags);
 void tdc_flush_unused_tables();
 TABLE *find_table_for_mdl_upgrade(THD *thd, const char *db,
                                   const char *table_name,
@@ -312,31 +354,9 @@ extern HASH table_def_cache;
 
   @param table        TABLE structure pointer (which should be setup)
   @param table_list   TABLE_LIST structure pointer (owner of TABLE)
-  @param tablenr     table number
+  @param tableno      table number
 */
 
-
-inline void setup_table_map(TABLE *table, TABLE_LIST *table_list, uint tablenr)
-{
-  table->used_fields= 0;
-  table->const_table= 0;
-  table->null_row= 0;
-  table->status= STATUS_GARBAGE | STATUS_NOT_FOUND;
-  table->maybe_null= table_list->outer_join;
-  TABLE_LIST *embedding= table_list->embedding;
-  while (!table->maybe_null && embedding)
-  {
-    table->maybe_null= embedding->outer_join;
-    embedding= embedding->embedding;
-  }
-  table->tablenr= tablenr;
-  table->map= (table_map) 1 << tablenr;
-  table->force_index= table_list->force_index;
-  table->force_index_order= table->force_index_group= 0;
-  table->covering_keys= table->s->keys_for_keyread;
-  table->merge_keys.clear_all();
-}
-
 inline TABLE_LIST *find_table_in_global_list(TABLE_LIST *table,
                                              const char *db_name,
                                              const char *table_name)
@@ -354,20 +374,6 @@ inline TABLE_LIST *find_table_in_local_list(TABLE_LIST *table,
 }
 
 
-inline bool setup_fields_with_no_wrap(THD *thd, Ref_ptr_array ref_pointer_array,
-                                      List<Item> &item,
-                                      enum_mark_columns mark_used_columns,
-                                      List<Item> *sum_func_list,
-                                      bool allow_sum_func)
-{
-  bool res;
-  thd->lex->select_lex.no_wrap_view_item= TRUE;
-  res= setup_fields(thd, ref_pointer_array, item, mark_used_columns,
-                    sum_func_list, allow_sum_func);
-  thd->lex->select_lex.no_wrap_view_item= FALSE;
-  return res;
-}
-
 /**
   An abstract class for a strategy specifying how the prelocking
   algorithm should extend the prelocking set while processing
@@ -462,14 +468,12 @@ inline TABLE *open_n_lock_single_table(THD *thd, TABLE_LIST *table_l,
 }
 
 
-/* open_and_lock_tables with derived handling */
-inline bool open_and_lock_tables(THD *thd, TABLE_LIST *tables,
-                                 bool derived, uint flags)
+// open_and_lock_tables with default prelocking strategy
+inline bool open_and_lock_tables(THD *thd, TABLE_LIST *tables, uint flags)
 {
   DML_prelocking_strategy prelocking_strategy;
 
-  return open_and_lock_tables(thd, tables, derived, flags,
-                              &prelocking_strategy);
+  return open_and_lock_tables(thd, tables, flags, &prelocking_strategy);
 }
 
 
@@ -530,6 +534,11 @@ public:
     return m_has_protection_against_grl;
   }
 
+  bool can_back_off() const
+  {
+    return !m_has_locks;
+  }
+
 private:
   /* THD for which tables are opened. */
   THD *m_thd;
@@ -569,7 +578,7 @@ private:
 
 inline bool is_temporary_table(TABLE_LIST *tl)
 {
-  if (tl->view || tl->schema_table)
+  if (tl->is_view() || tl->schema_table)
     return FALSE;
 
   if (!tl->table)
@@ -586,6 +595,70 @@ inline bool is_temporary_table(TABLE_LIST *tl)
   return tl->table->s->tmp_table != NO_TMP_TABLE;
 }
 
+/**
+  A simple holder for Internal_error_handler.
+  The class utilizes RAII technique to not forget to pop the handler.
+
+  @tparam Error_handler      Internal_error_handler to instantiate.
+  @tparam Error_handler_arg  Type of the error handler ctor argument.
+*/
+template<typename Error_handler, typename Error_handler_arg>
+class Internal_error_handler_holder
+{
+  THD *m_thd;
+  bool m_activate;
+  Error_handler m_error_handler;
+
+public:
+  Internal_error_handler_holder(THD *thd, bool activate,
+                                Error_handler_arg *arg)
+    : m_thd(thd), m_activate(activate), m_error_handler(arg)
+  {
+    if (activate)
+      thd->push_internal_handler(&m_error_handler);
+  }
+
+
+  ~Internal_error_handler_holder()
+  {
+    if (m_activate)
+      m_thd->pop_internal_handler();
+  }
+};
+
+/**
+   An Internal_error_handler that suppresses errors regarding views'
+   underlying tables that occur during privilege checking. It hides errors which
+   show view underlying table information.
+   This happens in the cases when
+
+   - A view's underlying table (e.g. referenced in its SELECT list) does not
+     exist or columns of underlying table are altered. There should not be an
+     error as no attempt was made to access it per se.
+
+   - Access is denied for some table, column, function or stored procedure
+     such as mentioned above. This error gets raised automatically, since we
+     can't untangle its access checking from that of the view itself.
+
+    There are currently two mechanisms at work that handle errors for views
+    based on an Internal_error_handler. This one and another one is
+    Show_create_error_handler. The latter handles errors encountered during
+    execution of SHOW CREATE VIEW, while this mechanism using this method is
+    handles SELECT from views. The two methods should not clash.
+
+*/
+class View_error_handler : public Internal_error_handler
+{
+  TABLE_LIST *m_top_view;
+
+public:
+  View_error_handler(TABLE_LIST *top_view) :
+  m_top_view(top_view)
+  {}
+  virtual bool handle_condition(THD *thd, uint sql_errno, const char *,
+                                Sql_condition::enum_severity_level *level,
+                                const char *message);
+};
 
 /**
   This internal handler is used to trap ER_NO_SUCH_TABLE.
@@ -598,22 +671,96 @@ public:
     : m_handled_errors(0), m_unhandled_errors(0)
   {}
 
-  bool handle_condition(THD *thd,
-                        uint sql_errno,
-                        const char* sqlstate,
-                        Sql_condition::enum_warning_level level,
-                        const char* msg,
-                        Sql_condition ** cond_hdl);
+  virtual bool handle_condition(THD *thd,
+                                uint sql_errno,
+                                const char* sqlstate,
+                                Sql_condition::enum_severity_level *level,
+                                const char* msg)
+  {
+    if (sql_errno == ER_NO_SUCH_TABLE)
+    {
+      m_handled_errors++;
+      return true;
+    }
+
+    m_unhandled_errors++;
+    return false;
+  }
 
   /**
-    Returns TRUE if one or more ER_NO_SUCH_TABLE errors have been
-    trapped and no other errors have been seen. FALSE otherwise.
+    Returns true if one or more ER_NO_SUCH_TABLE errors have been
+    trapped and no other errors have been seen. false otherwise.
   */
-  bool safely_trapped_errors();
+  bool safely_trapped_errors() const
+  {
+    /*
+      If m_unhandled_errors != 0, something else, unanticipated, happened,
+      so the error is not trapped but returned to the caller.
+      Multiple ER_NO_SUCH_TABLE can be raised in case of views.
+    */
+    return ((m_handled_errors > 0) && (m_unhandled_errors == 0));
+  }
 
 private:
   int m_handled_errors;
   int m_unhandled_errors;
 };
 
+
+/**
+  This internal handler implements downgrade from SL_ERROR to SL_WARNING
+  for statements which support IGNORE.
+*/
+
+class Ignore_error_handler : public Internal_error_handler
+{
+public:
+  virtual bool handle_condition(THD *thd,
+                                uint sql_errno,
+                                const char* sqlstate,
+                                Sql_condition::enum_severity_level *level,
+                                const char* msg);
+};
+
+/**
+  This internal handler implements upgrade from SL_WARNING to SL_ERROR
+  for the error codes affected by STRICT mode. Currently STRICT mode does
+  not affect SELECT statements.
+*/
+
+class Strict_error_handler : public Internal_error_handler
+{
+public:
+  enum enum_set_select_behavior
+  {
+    DISABLE_SET_SELECT_STRICT_ERROR_HANDLER,
+    ENABLE_SET_SELECT_STRICT_ERROR_HANDLER
+  };
+
+  Strict_error_handler()
+    : m_set_select_behavior(DISABLE_SET_SELECT_STRICT_ERROR_HANDLER)
+  {}
+
+  Strict_error_handler(enum_set_select_behavior param)
+    : m_set_select_behavior(param)
+  {}
+
+  virtual bool handle_condition(THD *thd,
+                                uint sql_errno,
+                                const char* sqlstate,
+                                Sql_condition::enum_severity_level *level,
+                                const char* msg);
+
+private:
+  /*
+    For SELECT and SET statement, we do not always give error in STRICT mode.
+    For triggers, Strict_error_handler is pushed in the beginning of statement.
+    If a SELECT or SET is executed from the Trigger, it should not always give
+    error. We use this flag to choose when to give error and when warning.
+  */
+  enum_set_select_behavior m_set_select_behavior;
+};
+
+void update_indexed_column_map(TABLE *table, MY_BITMAP *read_set);
+
 #endif /* SQL_BASE_INCLUDED */
