@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,10 +14,9 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "sql_reload.h"
-#include "sql_priv.h"
 #include "mysqld.h"      // select_errors
 #include "sql_class.h"   // THD
-#include "sql_acl.h"     // acl_reload
+#include "auth_common.h" // acl_reload, grant_reload
 #include "sql_servers.h" // servers_reload
 #include "sql_connect.h" // reset_mqh
 #include "sql_base.h"    // close_cached_tables
@@ -27,8 +26,10 @@
 #include "rpl_slave.h"   // reset_slave
 #include "rpl_rli.h"     // rotate_relay_log
 #include "rpl_mi.h"
+#include "rpl_msr.h"     /* multisource replication */
 #include "debug_sync.h"
-#include "des_key_file.h"
+#include "connection_handler_impl.h"
+#include "opt_costconstantcache.h"     // reload_optimizer_cost_constants
 
 
 /**
@@ -97,8 +98,6 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
     if (tmp_thd)
     {
       delete tmp_thd;
-      /* Remember that we don't have a THD */
-      my_pthread_setspecific_ptr(THR_THD,  0);
       thd= 0;
     }
     reset_mqh((LEX_USER *)NULL, TRUE);
@@ -132,12 +131,10 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
     }
 
   if ((options & REFRESH_SLOW_LOG) && opt_slow_log)
-    if (logger.flush_slow_log())
-      result= 1;
+    query_logger.reopen_log_file(QUERY_LOG_SLOW);
 
-  if ((options & REFRESH_GENERAL_LOG) && opt_log)
-    if (logger.flush_general_log())
-      result= 1;
+  if ((options & REFRESH_GENERAL_LOG) && opt_general_log)
+    query_logger.reopen_log_file(QUERY_LOG_GENERAL);
 
   if (options & REFRESH_ENGINE_LOG)
     if (ha_flush_logs(NULL))
@@ -173,26 +170,18 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
     if (options & REFRESH_RELAY_LOG)
     {
 #ifdef HAVE_REPLICATION
-      mysql_mutex_lock(&LOCK_active_mi);
-      if (active_mi != NULL)
-      {
-        mysql_mutex_lock(&active_mi->data_lock);
-        if (rotate_relay_log(active_mi))
-          *write_to_binlog= -1;
-        mysql_mutex_unlock(&active_mi->data_lock);
-      }
-      mysql_mutex_unlock(&LOCK_active_mi);
+      if (flush_relay_logs_cmd(thd))
+        *write_to_binlog= -1;
 #endif
     }
     if (tmp_thd)
     {
       delete tmp_thd;
       /* Remember that we don't have a THD */
-      my_pthread_setspecific_ptr(THR_THD,  0);
+      my_thread_set_THR_THD(NULL);
       thd= 0;
     }
   }
-#ifdef HAVE_QUERY_CACHE
   if (options & REFRESH_QUERY_CACHE_FREE)
   {
     query_cache.pack();				// FLUSH QUERY CACHE
@@ -202,11 +191,11 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
   {
     query_cache.flush();			// RESET QUERY CACHE
   }
-#endif /*HAVE_QUERY_CACHE*/
 
   DBUG_ASSERT(!thd || thd->locked_tables_mode ||
               !thd->mdl_context.has_locks() ||
               thd->handler_tables_hash.records ||
+              thd->ull_hash.records ||
               thd->global_read_lock.is_acquired());
 
   /*
@@ -276,8 +265,8 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
             global read lock.
           */
           if (thd->open_tables &&
-              !thd->mdl_context.is_lock_owner(MDL_key::GLOBAL, "", "",
-                                              MDL_INTENTION_EXCLUSIVE))
+              !thd->mdl_context.owns_equal_or_stronger_lock(MDL_key::GLOBAL,
+                                  "", "", MDL_INTENTION_EXCLUSIVE))
           {
             my_error(ER_TABLE_NOT_LOCKED_FOR_WRITE, MYF(0),
                      thd->open_tables->s->table_name.str);
@@ -314,8 +303,10 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
     hostname_cache_refresh();
   if (thd && (options & REFRESH_STATUS))
     refresh_status(thd);
+#ifndef EMBEDDED_LIBRARY
   if (options & REFRESH_THREADS)
-    kill_blocked_pthreads();
+    Per_thread_connection_handler::kill_blocked_pthreads();
+#endif
 #ifdef HAVE_REPLICATION
   if (options & REFRESH_MASTER)
   {
@@ -328,7 +319,7 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
     }
   }
 #endif
-#ifdef HAVE_OPENSSL
+#ifdef OPENSSL
    if (options & REFRESH_DES_KEY_FILE)
    {
      if (des_key_file && load_des_key_file(des_key_file))
@@ -338,22 +329,17 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
      }
    }
 #endif
+  if (options & REFRESH_OPTIMIZER_COSTS)
+    reload_optimizer_cost_constants();
 #ifdef HAVE_REPLICATION
  if (options & REFRESH_SLAVE)
  {
    tmp_write_to_binlog= 0;
-   mysql_mutex_lock(&LOCK_active_mi);
-   if (active_mi != NULL && reset_slave(thd, active_mi))
+   if (reset_slave_cmd(thd))
    {
-     /* NOTE: my_error() has been already called by reset_slave(). */
+     /*NOTE: my_error() has been already called by reset_slave() */
      result= 1;
    }
-   else if (active_mi == NULL)
-   {
-     result= 1;
-     my_error(ER_SLAVE_CONFIGURATION, MYF(0));
-   }
-   mysql_mutex_unlock(&LOCK_active_mi);
  }
 #endif
  if (options & REFRESH_USER_RESOURCES)
@@ -490,8 +476,7 @@ bool flush_tables_with_read_lock(THD *thd, TABLE_LIST *all_tables)
     acquire SNW locks to ensure that they can be locked for
     read without further waiting.
   */
-  if (open_and_lock_tables(thd, all_tables, FALSE,
-                           MYSQL_OPEN_SKIP_SCOPED_MDL_LOCK,
+  if (open_and_lock_tables(thd, all_tables, MYSQL_OPEN_SKIP_SCOPED_MDL_LOCK,
                            &lock_tables_prelocking_strategy) ||
       thd->locked_tables_list.init_locked_tables(thd))
   {
@@ -550,9 +535,12 @@ bool flush_tables_for_export(THD *thd, TABLE_LIST *all_tables)
     Acquire SNW locks on tables to be exported. Don't acquire
     global IX as this will make this statement incompatible
     with FLUSH TABLES WITH READ LOCK.
+    We can't acquire SRO locks instead of SNW locks as it will
+    make two concurrent FLUSH TABLE ... FOR EXPORT statements
+    for the same table possible, which creates race between
+    creation/deletion of metadata file.
   */
-  if (open_and_lock_tables(thd, all_tables, false,
-                           MYSQL_OPEN_SKIP_SCOPED_MDL_LOCK,
+  if (open_and_lock_tables(thd, all_tables, MYSQL_OPEN_SKIP_SCOPED_MDL_LOCK,
                            &lock_tables_prelocking_strategy))
   {
     return true;
