@@ -1,20 +1,14 @@
-/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; version 2 of
+   the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -38,15 +32,16 @@
 */
 
 
-#include "sql_acl.h"    // append_user
+#include "auth_common.h"    // append_user
 #include "sql_parse.h"  // get_current_user
 #include "sql_show.h"   // append_identifier
 #include "sp_head.h"    // struct set_var_base
 #include "rpl_slave.h"  // SLAVE_SQL, SLAVE_IO
+#include "mysqld.h"     // opt_log_backward_compatible_user_definitions
 
 
 /**
-  Append a key/value pair to a string, with an optional preceeding comma.
+  Append a key/value pair to a string, with an optional preceding comma.
   For numeric values.
 
   @param           str                  The string to append to
@@ -78,7 +73,7 @@ bool append_int(String *str, bool comma, const char *txt, size_t len,
 
 /**
   Append a key/value pair to a string if the value is non-NULL,
-  with an optional preceeding comma.
+  with an optional preceding comma.
 
   @param           str                  The string to append to
   @param           comma                Prepend a comma?
@@ -88,7 +83,7 @@ bool append_int(String *str, bool comma, const char *txt, size_t len,
   @retval          false if any subsequent key/value pair would be the first
 */
 
-bool append_str(String *str, bool comma, const char *key, char *val)
+bool append_str(String *str, bool comma, const char *key, const char *val)
 {
   if (val)
   {
@@ -103,6 +98,91 @@ bool append_str(String *str, bool comma, const char *key, char *val)
   return comma;
 }
 
+void rewrite_ssl_properties(LEX *lex, String *rlb)
+{
+  if (lex->ssl_type != SSL_TYPE_NOT_SPECIFIED)
+  {
+    rlb->append(STRING_WITH_LEN(" REQUIRE"));
+    switch (lex->ssl_type)
+    {
+    case SSL_TYPE_SPECIFIED:
+      if (lex->x509_subject)
+      {
+        rlb->append(STRING_WITH_LEN(" SUBJECT '"));
+        rlb->append(lex->x509_subject);
+        rlb->append(STRING_WITH_LEN("'"));
+      }
+      if (lex->x509_issuer)
+      {
+        rlb->append(STRING_WITH_LEN(" ISSUER '"));
+        rlb->append(lex->x509_issuer);
+        rlb->append(STRING_WITH_LEN("'"));
+      }
+      if (lex->ssl_cipher)
+      {
+        rlb->append(STRING_WITH_LEN(" CIPHER '"));
+        rlb->append(lex->ssl_cipher);
+        rlb->append(STRING_WITH_LEN("'"));
+      }
+      break;
+    case SSL_TYPE_X509:
+      rlb->append(STRING_WITH_LEN(" X509"));
+      break;
+    case SSL_TYPE_ANY:
+      rlb->append(STRING_WITH_LEN(" SSL"));
+      break;
+    case SSL_TYPE_NOT_SPECIFIED:
+      /* fall-thru */
+    case SSL_TYPE_NONE:
+      rlb->append(STRING_WITH_LEN(" NONE"));
+      break;
+    }
+  }
+}
+
+void rewrite_user_resources(LEX *lex, String *rlb)
+{
+  if (lex->mqh.specified_limits || (lex->grant & GRANT_ACL))
+  {
+    rlb->append(STRING_WITH_LEN(" WITH"));
+    if (lex->grant & GRANT_ACL)
+      rlb->append(STRING_WITH_LEN(" GRANT OPTION"));
+
+    append_int(rlb, false, STRING_WITH_LEN(" MAX_QUERIES_PER_HOUR "),
+               lex->mqh.questions,
+               lex->mqh.specified_limits & USER_RESOURCES::QUERIES_PER_HOUR);
+
+    append_int(rlb, false, STRING_WITH_LEN(" MAX_UPDATES_PER_HOUR "),
+               lex->mqh.updates,
+               lex->mqh.specified_limits & USER_RESOURCES::UPDATES_PER_HOUR);
+
+    append_int(rlb, false, STRING_WITH_LEN(" MAX_CONNECTIONS_PER_HOUR "),
+               lex->mqh.conn_per_hour,
+               lex->mqh.specified_limits & USER_RESOURCES::CONNECTIONS_PER_HOUR);
+
+    append_int(rlb, false, STRING_WITH_LEN(" MAX_USER_CONNECTIONS "),
+               lex->mqh.user_conn,
+               lex->mqh.specified_limits & USER_RESOURCES::USER_CONNECTIONS);
+  }
+}
+
+void rewrite_account_lock(LEX *lex, String *rlb, bool lock_only)
+{
+  if (!lex->alter_password.account_locked)
+  {
+    if (lock_only)
+    {
+      /* Do not write account enable state. */
+      return;
+    }
+
+    rlb->append(STRING_WITH_LEN(" ACCOUNT UNLOCK"));
+  }
+  else
+  {
+    rlb->append(STRING_WITH_LEN(" ACCOUNT LOCK"));
+  }
+}
 
 /**
   Rewrite a GRANT statement.
@@ -111,19 +191,17 @@ bool append_str(String *str, bool comma, const char *key, char *val)
   @param rlb      An empty String object to put the rewritten query in.
 */
 
-static void mysql_rewrite_grant(THD *thd, String *rlb)
+void mysql_rewrite_grant(THD *thd, String *rlb)
 {
   LEX        *lex= thd->lex;
-  TABLE_LIST *first_table= (TABLE_LIST*) lex->select_lex.table_list.first;
+  TABLE_LIST *first_table= (TABLE_LIST*) lex->select_lex->table_list.first;
   bool        comma= FALSE, comma_inner;
-  bool        proxy_grant= lex->type == TYPE_ENUM_PROXY;
   String      cols(1024);
   int         c;
 
   rlb->append(STRING_WITH_LEN("GRANT "));
-  if (proxy_grant)
-    rlb->append(STRING_WITH_LEN("PROXY"));
-  else if (lex->all_privileges)
+
+  if (lex->all_privileges)
     rlb->append(STRING_WITH_LEN("ALL PRIVILEGES"));
   else
   {
@@ -187,29 +265,29 @@ static void mysql_rewrite_grant(THD *thd, String *rlb)
   default:                                                              break;
   }
 
-  LEX_USER *user_name, *tmp_user_name;
-  List_iterator <LEX_USER> user_list(lex->users_list);
-  comma= FALSE;
-
-  if (proxy_grant)
+  if (first_table)
   {
-    tmp_user_name= user_list++;
-    user_name= get_current_user(thd, tmp_user_name);
-    if (user_name)
-      append_user(thd, rlb, user_name, comma, true);
-  }
-  else if (first_table)
-  {
-    append_identifier(thd, rlb, first_table->db, strlen(first_table->db));
-    rlb->append(STRING_WITH_LEN("."));
-    append_identifier(thd, rlb, first_table->table_name,
-                      strlen(first_table->table_name));
+    if (first_table->is_view())
+    {
+      append_identifier(thd, rlb, first_table->view_db.str,
+                        first_table->view_db.length);
+      rlb->append(STRING_WITH_LEN("."));
+      append_identifier(thd, rlb, first_table->view_name.str,
+                        first_table->view_name.length);
+    }
+    else
+    {
+      append_identifier(thd, rlb, first_table->db, strlen(first_table->db));
+      rlb->append(STRING_WITH_LEN("."));
+      append_identifier(thd, rlb, first_table->table_name,
+                        strlen(first_table->table_name));
+    }
   }
   else
   {
-    if (lex->current_select->db)
-      append_identifier(thd, rlb, lex->current_select->db,
-                        strlen(lex->current_select->db));
+    if (lex->current_select()->db)
+      append_identifier(thd, rlb, lex->current_select()->db,
+                        strlen(lex->current_select()->db));
     else
       rlb->append("*");
     rlb->append(STRING_WITH_LEN(".*"));
@@ -217,77 +295,24 @@ static void mysql_rewrite_grant(THD *thd, String *rlb)
 
   rlb->append(STRING_WITH_LEN(" TO "));
   {
+    LEX_USER *user_name, *tmp_user_name;
+    List_iterator <LEX_USER> user_list(lex->users_list);
+    bool comma= FALSE;
+
     while ((tmp_user_name= user_list++))
     {
       if ((user_name= get_current_user(thd, tmp_user_name)))
       {
-        append_user(thd, rlb, user_name, comma, true);
+        if (opt_log_backward_compatible_user_definitions)
+          append_user(thd, rlb, user_name, comma, true);
+        else
+          append_user_new(thd, rlb, user_name, comma);
         comma= TRUE;
       }
     }
   }
-
-  if (lex->ssl_type != SSL_TYPE_NOT_SPECIFIED)
-  {
-    rlb->append(STRING_WITH_LEN(" REQUIRE"));
-    switch (lex->ssl_type)
-    {
-    case SSL_TYPE_SPECIFIED:
-      if (lex->x509_subject)
-      {
-        rlb->append(STRING_WITH_LEN(" SUBJECT '"));
-        rlb->append(lex->x509_subject);
-        rlb->append(STRING_WITH_LEN("'"));
-      }
-      if (lex->x509_issuer)
-      {
-        rlb->append(STRING_WITH_LEN(" ISSUER '"));
-        rlb->append(lex->x509_issuer);
-        rlb->append(STRING_WITH_LEN("'"));
-      }
-      if (lex->ssl_cipher)
-      {
-        rlb->append(STRING_WITH_LEN(" CIPHER '"));
-        rlb->append(lex->ssl_cipher);
-        rlb->append(STRING_WITH_LEN("'"));
-      }
-      break;
-    case SSL_TYPE_X509:
-      rlb->append(STRING_WITH_LEN(" X509"));
-      break;
-    case SSL_TYPE_ANY:
-      rlb->append(STRING_WITH_LEN(" SSL"));
-      break;
-    case SSL_TYPE_NOT_SPECIFIED:
-      /* fall-thru */
-    case SSL_TYPE_NONE:
-      rlb->append(STRING_WITH_LEN(" NONE"));
-      break;
-    }
-  }
-
-  if (lex->mqh.specified_limits || (lex->grant & GRANT_ACL))
-  {
-    rlb->append(STRING_WITH_LEN(" WITH"));
-    if (lex->grant & GRANT_ACL)
-      rlb->append(STRING_WITH_LEN(" GRANT OPTION"));
-
-    append_int(rlb, false, STRING_WITH_LEN(" MAX_QUERIES_PER_HOUR "),
-               lex->mqh.questions,
-               lex->mqh.specified_limits & USER_RESOURCES::QUERIES_PER_HOUR);
-
-    append_int(rlb, false, STRING_WITH_LEN(" MAX_UPDATES_PER_HOUR "),
-               lex->mqh.updates,
-               lex->mqh.specified_limits & USER_RESOURCES::UPDATES_PER_HOUR);
-
-    append_int(rlb, false, STRING_WITH_LEN(" MAX_CONNECTIONS_PER_HOUR "),
-               lex->mqh.conn_per_hour,
-               lex->mqh.specified_limits & USER_RESOURCES::CONNECTIONS_PER_HOUR);
-
-    append_int(rlb, false, STRING_WITH_LEN(" MAX_USER_CONNECTIONS "),
-               lex->mqh.user_conn,
-               lex->mqh.specified_limits & USER_RESOURCES::USER_CONNECTIONS);
-  }
+  rewrite_ssl_properties(lex, rlb);
+  rewrite_user_resources(lex, rlb);
 }
 
 
@@ -318,32 +343,60 @@ static void mysql_rewrite_set(THD *thd, String *rlb)
   }
 }
 
-
 /**
-  Rewrite CREATE USER statement.
+  Rewrite CREATE/ALTER USER statement.
 
   @param thd      The THD to rewrite for.
   @param rlb      An empty String object to put the rewritten query in.
 */
 
-static void mysql_rewrite_create_user(THD *thd, String *rlb)
+void mysql_rewrite_create_alter_user(THD *thd, String *rlb)
 {
   LEX                      *lex= thd->lex;
   LEX_USER                 *user_name, *tmp_user_name;
   List_iterator <LEX_USER>  user_list(lex->users_list);
   bool                      comma= FALSE;
 
-  rlb->append(STRING_WITH_LEN("CREATE USER "));
+  if (thd->lex->sql_command == SQLCOM_CREATE_USER ||
+      thd->lex->sql_command == SQLCOM_SHOW_CREATE_USER)
+    rlb->append(STRING_WITH_LEN("CREATE USER "));
+  else
+    rlb->append(STRING_WITH_LEN("ALTER USER "));
+
   while ((tmp_user_name= user_list++))
   {
     if ((user_name= get_current_user(thd, tmp_user_name)))
     {
-      append_user(thd, rlb, user_name, comma, TRUE);
+      if (opt_log_backward_compatible_user_definitions)
+        append_user(thd, rlb, user_name, comma, true);
+      else
+        append_user_new(thd, rlb, user_name, comma);
       comma= TRUE;
     }
   }
-}
 
+  rewrite_ssl_properties(lex, rlb);
+  rewrite_user_resources(lex, rlb);
+
+  /* rewrite password expired */
+  if (lex->alter_password.update_password_expired_column)
+    rlb->append(STRING_WITH_LEN(" PASSWORD EXPIRE"));
+  else if (lex->alter_password.expire_after_days)
+  {
+    append_int(rlb, false, STRING_WITH_LEN(" PASSWORD EXPIRE INTERVAL "),
+               lex->alter_password.expire_after_days, TRUE);
+    rlb->append(STRING_WITH_LEN(" DAY"));
+  }
+  else if (lex->alter_password.use_default_password_lifetime)
+  {
+    if (!opt_log_backward_compatible_user_definitions)
+      rlb->append(STRING_WITH_LEN(" PASSWORD EXPIRE DEFAULT"));
+  }
+  else
+    rlb->append(STRING_WITH_LEN(" PASSWORD EXPIRE NEVER"));
+
+  rewrite_account_lock(lex, rlb, false);
+}
 
 /**
   Rewrite a CHANGE MASTER statement.
@@ -451,10 +504,9 @@ static void mysql_rewrite_change_master(THD *thd, String *rlb)
   {
     bool first= TRUE;
     rlb->append(STRING_WITH_LEN(" IGNORE_SERVER_IDS = ( "));
-    for (uint i= 0; i < lex->mi.repl_ignore_server_ids.elements; i++)
+    for (size_t i= 0; i < lex->mi.repl_ignore_server_ids.size(); i++)
     {
-      ulong s_id;
-      get_dynamic(&lex->mi.repl_ignore_server_ids, (uchar*) &s_id, i);
+      ulong s_id= lex->mi.repl_ignore_server_ids[i];
       if (first)
         first= FALSE;
       else
@@ -471,7 +523,7 @@ static void mysql_rewrite_change_master(THD *thd, String *rlb)
     else
     {
       char buf[64];
-      snprintf(buf, 64, "%f", lex->mi.heartbeat_period);
+      my_snprintf(buf, 64, "%f", lex->mi.heartbeat_period);
       rlb->append(buf);
     }
   }
@@ -498,7 +550,7 @@ static void mysql_rewrite_start_slave(THD *thd, String *rlb)
     rlb->append(STRING_WITH_LEN(" IO_THREAD"));
 
   /* we have printed the IO THREAD related options */
-  if (lex->slave_thd_opt & SLAVE_IO &&
+  if (lex->slave_thd_opt & SLAVE_IO && 
       lex->slave_thd_opt & SLAVE_SQL)
     rlb->append(STRING_WITH_LEN(","));
 
@@ -569,13 +621,14 @@ static void mysql_rewrite_server_options(THD *thd, String *rlb)
   rlb->append(STRING_WITH_LEN(" OPTIONS ( "));
 
   rlb->append(STRING_WITH_LEN("PASSWORD '<secret>'"));
-  append_str(rlb, true, "USER", lex->server_options.username);
-  append_str(rlb, true, "HOST", lex->server_options.host);
-  append_str(rlb, true, "DATABASE", lex->server_options.db);
-  append_str(rlb, true, "OWNER", lex->server_options.owner);
-  append_str(rlb, true, "SOCKET", lex->server_options.socket);
-  append_int(rlb, true, STRING_WITH_LEN("PORT "), lex->server_options.port,
-             lex->server_options.port > 0);
+  append_str(rlb, true, "USER", lex->server_options.get_username());
+  append_str(rlb, true, "HOST", lex->server_options.get_host());
+  append_str(rlb, true, "DATABASE", lex->server_options.get_db());
+  append_str(rlb, true, "OWNER", lex->server_options.get_owner());
+  append_str(rlb, true, "SOCKET", lex->server_options.get_socket());
+  append_int(rlb, true, STRING_WITH_LEN("PORT "),
+             lex->server_options.get_port(),
+             lex->server_options.get_port() != Server_options::PORT_NOT_SET);
 
   rlb->append(STRING_WITH_LEN(" )"));
 }
@@ -592,17 +645,17 @@ static void mysql_rewrite_create_server(THD *thd, String *rlb)
 {
   LEX *lex= thd->lex;
 
-  if (!lex->server_options.password)
+  if (!lex->server_options.get_password())
     return;
 
   rlb->append(STRING_WITH_LEN("CREATE SERVER "));
 
-  rlb->append(lex->server_options.server_name ?
-              lex->server_options.server_name : "");
+  rlb->append(lex->server_options.m_server_name.str ?
+              lex->server_options.m_server_name.str : "");
 
   rlb->append(STRING_WITH_LEN(" FOREIGN DATA WRAPPER '"));
-  rlb->append(lex->server_options.scheme ?
-              lex->server_options.scheme : "");
+  rlb->append(lex->server_options.get_scheme() ?
+              lex->server_options.get_scheme() : "");
   rlb->append(STRING_WITH_LEN("'"));
 
   mysql_rewrite_server_options(thd, rlb);
@@ -620,13 +673,13 @@ static void mysql_rewrite_alter_server(THD *thd, String *rlb)
 {
   LEX *lex= thd->lex;
 
-  if (!lex->server_options.password)
+  if (!lex->server_options.get_password())
     return;
 
   rlb->append(STRING_WITH_LEN("ALTER SERVER "));
 
-  rlb->append(lex->server_options.server_name ?
-              lex->server_options.server_name : "");
+  rlb->append(lex->server_options.m_server_name.str ?
+              lex->server_options.m_server_name.str : "");
 
   mysql_rewrite_server_options(thd, rlb);
 }
@@ -670,7 +723,7 @@ void mysql_rewrite_query(THD *thd)
 {
   String *rlb= &thd->rewritten_query;
 
-  rlb->free();
+  rlb->mem_free();
 
   if (thd->lex->contains_plaintext_password)
   {
@@ -678,7 +731,9 @@ void mysql_rewrite_query(THD *thd)
     {
     case SQLCOM_GRANT:         mysql_rewrite_grant(thd, rlb);         break;
     case SQLCOM_SET_OPTION:    mysql_rewrite_set(thd, rlb);           break;
-    case SQLCOM_CREATE_USER:   mysql_rewrite_create_user(thd, rlb);   break;
+    case SQLCOM_CREATE_USER:
+    case SQLCOM_ALTER_USER:
+                        mysql_rewrite_create_alter_user(thd, rlb);    break;
     case SQLCOM_CHANGE_MASTER: mysql_rewrite_change_master(thd, rlb); break;
     case SQLCOM_SLAVE_START:   mysql_rewrite_start_slave(thd, rlb);   break;
     case SQLCOM_CREATE_SERVER: mysql_rewrite_create_server(thd, rlb); break;
