@@ -1,6 +1,6 @@
 /***********************************************************************
 
-Copyright (c) 1995, 2013, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2015, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2009, Percona Inc.
 
 Portions of this file contain modifications contributed and copyrighted
@@ -37,7 +37,7 @@ Created 10/21/1995 Heikki Tuuri
 
 #include "univ.i"
 
-#ifndef __WIN__
+#ifndef _WIN32
 #include <dirent.h>
 #include <sys/stat.h>
 #include <time.h>
@@ -46,9 +46,7 @@ Created 10/21/1995 Heikki Tuuri
 /** File node of a tablespace or the log data space */
 struct fil_node_t;
 
-extern ibool	os_has_said_disk_full;
-/** Flag: enable debug printout for asynchronous i/o */
-extern ibool	os_aio_print_debug;
+extern bool	os_has_said_disk_full;
 
 /** Number of pending os_file_pread() operations */
 extern ulint	os_file_n_pending_preads;
@@ -60,7 +58,7 @@ extern ulint	os_n_pending_reads;
 /** Number of pending write operations */
 extern ulint	os_n_pending_writes;
 
-#ifdef __WIN__
+#ifdef _WIN32
 
 /** We define always WIN_ASYNC_IO, and check at run-time whether
    the OS actually supports it: Win 95 does not, NT does. */
@@ -73,22 +71,24 @@ extern ulint	os_n_pending_writes;
 
 /** File offset in bytes */
 typedef ib_uint64_t os_offset_t;
-#ifdef __WIN__
+#ifdef _WIN32
 /** File handle */
 # define os_file_t	HANDLE
 /** Convert a C file descriptor to a native file handle
-@param fd	file descriptor
-@return		native file handle */
+@param fd file descriptor
+@return native file handle */
 # define OS_FILE_FROM_FD(fd) (HANDLE) _get_osfhandle(fd)
 #else
 /** File handle */
 typedef int	os_file_t;
 /** Convert a C file descriptor to a native file handle
-@param fd	file descriptor
-@return		native file handle */
+@param fd file descriptor
+@return native file handle */
 # define OS_FILE_FROM_FD(fd) fd
 #endif
 
+static const os_file_t OS_FILE_CLOSED = os_file_t(~0);
+
 /** Umask for creating files */
 extern ulint	os_innodb_umask;
 
@@ -136,6 +136,7 @@ enum os_file_create_t {
 /** Types for file create @{ */
 #define	OS_DATA_FILE			100
 #define OS_LOG_FILE			101
+#define OS_DATA_TEMP_FILE		102
 /* @} */
 
 /** Error codes from os_file_get_last_error @{ */
@@ -190,27 +191,15 @@ enum os_file_create_t {
 				effect only in simulated aio */
 /* @} */
 
-#define OS_WIN31	1	/*!< Microsoft Windows 3.x */
-#define OS_WIN95	2	/*!< Microsoft Windows 95 */
-#define OS_WINNT	3	/*!< Microsoft Windows NT 3.x */
-#define OS_WIN2000	4	/*!< Microsoft Windows 2000 */
-#define OS_WINXP	5	/*!< Microsoft Windows XP
-				or Windows Server 2003 */
-#define OS_WINVISTA	6	/*!< Microsoft Windows Vista
-				or Windows Server 2008 */
-#define OS_WIN7		7	/*!< Microsoft Windows 7
-				or Windows Server 2008 R2 */
-
-
 extern ulint	os_n_file_reads;
 extern ulint	os_n_file_writes;
 extern ulint	os_n_fsyncs;
 
 #ifdef UNIV_PFS_IO
 /* Keys to register InnoDB I/O with performance schema */
-extern mysql_pfs_key_t	innodb_file_data_key;
-extern mysql_pfs_key_t	innodb_file_log_key;
-extern mysql_pfs_key_t	innodb_file_temp_key;
+extern mysql_pfs_key_t	innodb_data_file_key;
+extern mysql_pfs_key_t	innodb_log_file_key;
+extern mysql_pfs_key_t	innodb_temp_file_key;
 
 /* Following four macros are instumentations to register
 various file I/O operations with performance schema.
@@ -294,26 +283,30 @@ os_file_write
 The wrapper functions have the prefix of "innodb_". */
 
 #ifdef UNIV_PFS_IO
-# define os_file_create(key, name, create, purpose, type, success)	\
+# define os_file_create(key, name, create, purpose, type, read_only,	\
+			success)					\
 	pfs_os_file_create_func(key, name, create, purpose,	type,	\
-				success, __FILE__, __LINE__)
+				read_only, success, __FILE__, __LINE__)
 
-# define os_file_create_simple(key, name, create, access, success)	\
+# define os_file_create_simple(key, name, create, access,		\
+		read_only, success)					\
 	pfs_os_file_create_simple_func(key, name, create, access,	\
-				       success, __FILE__, __LINE__)
+		read_only, success, __FILE__, __LINE__)
 
 # define os_file_create_simple_no_error_handling(			\
-		key, name, create_mode, access, success)		\
+		key, name, create_mode, access, read_only, success)	\
 	pfs_os_file_create_simple_no_error_handling_func(		\
-		key, name, create_mode, access, success, __FILE__, __LINE__)
+		key, name, create_mode, access,				\
+		read_only, success, __FILE__, __LINE__)
 
 # define os_file_close(file)						\
 	pfs_os_file_close_func(file, __FILE__, __LINE__)
 
 # define os_aio(type, mode, name, file, buf, offset,			\
-		n, message1, message2)					\
+		n, read_only, message1, message2)			\
 	pfs_os_aio_func(type, mode, name, file, buf, offset,		\
-			n, message1, message2, __FILE__, __LINE__)
+			n, read_only, message1, message2,		\
+			__FILE__, __LINE__)
 
 # define os_file_read(file, buf, offset, n)				\
 	pfs_os_file_read_func(file, buf, offset, n, __FILE__, __LINE__)
@@ -335,28 +328,33 @@ The wrapper functions have the prefix of "innodb_". */
 # define os_file_delete(key, name)					\
 	pfs_os_file_delete_func(key, name, __FILE__, __LINE__)
 
-# define os_file_delete_if_exists(key, name)				\
-	pfs_os_file_delete_if_exists_func(key, name, __FILE__, __LINE__)
+# define os_file_delete_if_exists(key, name, exist)			\
+	pfs_os_file_delete_if_exists_func(key, name, exist, __FILE__, __LINE__)
 #else /* UNIV_PFS_IO */
 
 /* If UNIV_PFS_IO is not defined, these I/O APIs point
 to original un-instrumented file I/O APIs */
-# define os_file_create(key, name, create, purpose, type, success)	\
-	os_file_create_func(name, create, purpose, type, success)
+# define os_file_create(key, name, create, purpose, type, read_only,	\
+			success)					\
+	os_file_create_func(name, create, purpose, type, read_only,	\
+			success)
 
-# define os_file_create_simple(key, name, create_mode, access, success)	\
-	os_file_create_simple_func(name, create_mode, access, success)
+# define os_file_create_simple(key, name, create_mode, access,		\
+		read_only, success)					\
+	os_file_create_simple_func(name, create_mode, access,		\
+		read_only, success)
 
 # define os_file_create_simple_no_error_handling(			\
-		key, name, create_mode, access, success)		\
+		key, name, create_mode, access, read_only, success)	\
 	os_file_create_simple_no_error_handling_func(			\
-		name, create_mode, access, success)
+		name, create_mode, access, read_only, success)
 
 # define os_file_close(file)	os_file_close_func(file)
 
-# define os_aio(type, mode, name, file, buf, offset, n, message1, message2) \
-	os_aio_func(type, mode, name, file, buf, offset, n,		\
-		    message1, message2)
+# define os_aio(type, mode, name, file, buf, offset,			\
+		n, read_only, message1, message2)			\
+	os_aio_func(type, mode, name, file, buf, offset,		\
+		n, read_only, message1, message2)
 
 # define os_file_read(file, buf, offset, n)	\
 	os_file_read_func(file, buf, offset, n)
@@ -374,8 +372,8 @@ to original un-instrumented file I/O APIs */
 
 # define os_file_delete(key, name)	os_file_delete_func(name)
 
-# define os_file_delete_if_exists(key, name)				\
-	os_file_delete_if_exists_func(name)
+# define os_file_delete_if_exists(key, name, exist)			\
+	os_file_delete_if_exists_func(name, exist)
 
 #endif /* UNIV_PFS_IO */
 
@@ -399,7 +397,7 @@ bigger than 4000 bytes */
 struct os_file_stat_t {
 	char		name[OS_FILE_MAX_PATH];	/*!< path to a file */
 	os_file_type_t	type;			/*!< file type */
-	ib_int64_t	size;			/*!< file size */
+	int64_t		size;			/*!< file size */
 	time_t		ctime;			/*!< creation time */
 	time_t		mtime;			/*!< modification time */
 	time_t		atime;			/*!< access time */
@@ -408,34 +406,22 @@ struct os_file_stat_t {
 						if type == OS_FILE_TYPE_FILE */
 };
 
-#ifdef __WIN__
+#ifdef _WIN32
 typedef HANDLE	os_file_dir_t;	/*!< directory stream */
 #else
 typedef DIR*	os_file_dir_t;	/*!< directory stream */
 #endif
 
-#ifdef __WIN__
-/***********************************************************************//**
-Gets the operating system version. Currently works only on Windows.
-@return	OS_WIN95, OS_WIN31, OS_WINNT, OS_WIN2000, OS_WINXP, OS_WINVISTA,
-OS_WIN7. */
-UNIV_INTERN
-ulint
-os_get_os_version(void);
-/*===================*/
-#endif /* __WIN__ */
 #ifndef UNIV_HOTBACKUP
 /****************************************************************//**
 Creates the seek mutexes used in positioned reads and writes. */
-UNIV_INTERN
 void
 os_io_init_simple(void);
 /*===================*/
 /***********************************************************************//**
 Creates a temporary file.  This function is like tmpfile(3), but
 the temporary file is created in the MySQL temporary directory.
-@return	temporary file handle, or NULL on error */
-
+@return temporary file handle, or NULL on error */
 FILE*
 os_file_create_tmpfile(void);
 /*========================*/
@@ -445,22 +431,20 @@ The os_file_opendir() function opens a directory stream corresponding to the
 directory named by the dirname argument. The directory stream is positioned
 at the first entry. In both Unix and Windows we automatically skip the '.'
 and '..' items at the start of the directory listing.
-@return	directory stream, NULL if error */
-UNIV_INTERN
+@return directory stream, NULL if error */
 os_file_dir_t
 os_file_opendir(
 /*============*/
 	const char*	dirname,	/*!< in: directory name; it must not
 					contain a trailing '\' or '/' */
-	ibool		error_is_fatal);/*!< in: TRUE if we should treat an
+	bool		error_is_fatal);/*!< in: TRUE if we should treat an
 					error as a fatal error; if we try to
 					open symlinks then we do not wish a
 					fatal error if it happens not to be
 					a directory */
 /***********************************************************************//**
 Closes a directory stream.
-@return	0 if success, -1 if failure */
-UNIV_INTERN
+@return 0 if success, -1 if failure */
 int
 os_file_closedir(
 /*=============*/
@@ -468,8 +452,7 @@ os_file_closedir(
 /***********************************************************************//**
 This function returns information of the next file in the directory. We jump
 over the '.' and '..' entries in the directory.
-@return	0 if ok, -1 if error, 1 if at the end of the directory */
-UNIV_INTERN
+@return 0 if ok, -1 if error, 1 if at the end of the directory */
 int
 os_file_readdir_next_file(
 /*======================*/
@@ -481,14 +464,13 @@ This function attempts to create a directory named pathname. The new directory
 gets default permissions. On Unix, the permissions are (0770 & ~umask). If the
 directory exists already, nothing is done and the call succeeds, unless the
 fail_if_exists arguments is true.
-@return	TRUE if call succeeds, FALSE on error */
-UNIV_INTERN
-ibool
+@return TRUE if call succeeds, FALSE on error */
+bool
 os_file_create_directory(
 /*=====================*/
 	const char*	pathname,	/*!< in: directory name as
 					null-terminated string */
-	ibool		fail_if_exists);/*!< in: if TRUE, pre-existing directory
+	bool		fail_if_exists);/*!< in: if TRUE, pre-existing directory
 					is treated as an error. */
 /****************************************************************//**
 NOTE! Use the corresponding macro os_file_create_simple(), not directly
@@ -496,7 +478,6 @@ this function!
 A simple function to open or create a file.
 @return own: handle to the file, not defined if error, error number
 can be retrieved with os_file_get_last_error */
-UNIV_INTERN
 os_file_t
 os_file_create_simple_func(
 /*=======================*/
@@ -505,14 +486,16 @@ os_file_create_simple_func(
 	ulint		create_mode,/*!< in: create mode */
 	ulint		access_type,/*!< in: OS_FILE_READ_ONLY or
 				OS_FILE_READ_WRITE */
-	ibool*		success);/*!< out: TRUE if succeed, FALSE if error */
+	bool		read_only_mode,
+				/*!< in: if true read only mode
+				checks are enforced. */
+	bool*		success);/*!< out: TRUE if succeed, FALSE if error */
 /****************************************************************//**
 NOTE! Use the corresponding macro
 os_file_create_simple_no_error_handling(), not directly this function!
 A simple function to open or create a file.
 @return own: handle to the file, not defined if error, error number
 can be retrieved with os_file_get_last_error */
-UNIV_INTERN
 os_file_t
 os_file_create_simple_no_error_handling_func(
 /*=========================================*/
@@ -523,11 +506,13 @@ os_file_create_simple_no_error_handling_func(
 				OS_FILE_READ_WRITE, or
 				OS_FILE_READ_ALLOW_DELETE; the last option is
 				used by a backup program reading the file */
-	ibool*		success)/*!< out: TRUE if succeed, FALSE if error */
+	bool		read_only_mode,
+				/*!< in: if true read only mode
+				checks are enforced. */
+	bool*		success)/*!< out: TRUE if succeed, FALSE if error */
 	__attribute__((nonnull, warn_unused_result));
 /****************************************************************//**
 Tries to disable OS caching on an opened file descriptor. */
-UNIV_INTERN
 void
 os_file_set_nocache(
 /*================*/
@@ -542,7 +527,6 @@ this function!
 Opens an existing file or creates a new.
 @return own: handle to the file, not defined if error, error number
 can be retrieved with os_file_get_last_error */
-UNIV_INTERN
 os_file_t
 os_file_create_func(
 /*================*/
@@ -557,12 +541,14 @@ os_file_create_func(
 				async i/o or unbuffered i/o: look in the
 				function source code for the exact rules */
 	ulint		type,	/*!< in: OS_DATA_FILE or OS_LOG_FILE */
-	ibool*		success)/*!< out: TRUE if succeed, FALSE if error */
+	bool		read_only_mode,
+				/*!< in: if true read only mode
+				checks are enforced. */
+	bool*		success)/*!< out: TRUE if succeed, FALSE if error */
 	__attribute__((nonnull, warn_unused_result));
 /***********************************************************************//**
 Deletes a file. The file has to be closed before calling this.
-@return	TRUE if success */
-UNIV_INTERN
+@return TRUE if success */
 bool
 os_file_delete_func(
 /*================*/
@@ -571,21 +557,20 @@ os_file_delete_func(
 
 /***********************************************************************//**
 Deletes a file if it exists. The file has to be closed before calling this.
-@return	TRUE if success */
-UNIV_INTERN
+@return TRUE if success */
 bool
 os_file_delete_if_exists_func(
 /*==========================*/
-	const char*	name);	/*!< in: file path as a null-terminated
+	const char*	name,	/*!< in: file path as a null-terminated
 				string */
+	bool*		exist);	/*!< out: indicate if file pre-exist */
 /***********************************************************************//**
 NOTE! Use the corresponding macro os_file_rename(), not directly
 this function!
 Renames a file (can also move it to another directory). It is safest that the
 file is closed before calling this function.
-@return	TRUE if success */
-UNIV_INTERN
-ibool
+@return TRUE if success */
+bool
 os_file_rename_func(
 /*================*/
 	const char*	oldpath,	/*!< in: old file path as a
@@ -596,9 +581,8 @@ NOTE! Use the corresponding macro os_file_close(), not directly this
 function!
 Closes a file handle. In case of error, error number can be retrieved with
 os_file_get_last_error.
-@return	TRUE if success */
-UNIV_INTERN
-ibool
+@return TRUE if success */
+bool
 os_file_close_func(
 /*===============*/
 	os_file_t	file);	/*!< in, own: handle to a file */
@@ -621,7 +605,10 @@ pfs_os_file_create_simple_func(
 	ulint		create_mode,/*!< in: create mode */
 	ulint		access_type,/*!< in: OS_FILE_READ_ONLY or
 				OS_FILE_READ_WRITE */
-	ibool*		success,/*!< out: TRUE if succeed, FALSE if error */
+	bool		read_only_mode,
+				/*!< in: if true read only mode
+				checks are enforced. */
+	bool*		success,/*!< out: TRUE if succeed, FALSE if error */
 	const char*	src_file,/*!< in: file name where func invoked */
 	ulint		src_line)/*!< in: line where the func invoked */
 	__attribute__((nonnull, warn_unused_result));
@@ -646,7 +633,10 @@ pfs_os_file_create_simple_no_error_handling_func(
 				OS_FILE_READ_WRITE, or
 				OS_FILE_READ_ALLOW_DELETE; the last option is
 				used by a backup program reading the file */
-	ibool*		success,/*!< out: TRUE if succeed, FALSE if error */
+	bool		read_only_mode,
+				/*!< in: if true read only mode
+				checks are enforced. */
+	bool*		success,/*!< out: TRUE if succeed, FALSE if error */
 	const char*	src_file,/*!< in: file name where func invoked */
 	ulint		src_line)/*!< in: line where the func invoked */
 	__attribute__((nonnull, warn_unused_result));
@@ -674,7 +664,10 @@ pfs_os_file_create_func(
 				async i/o or unbuffered i/o: look in the
 				function source code for the exact rules */
 	ulint		type,	/*!< in: OS_DATA_FILE or OS_LOG_FILE */
-	ibool*		success,/*!< out: TRUE if succeed, FALSE if error */
+	bool		read_only_mode,
+				/*!< in: if true read only mode
+				checks are enforced. */
+	bool*		success,/*!< out: TRUE if succeed, FALSE if error */
 	const char*	src_file,/*!< in: file name where func invoked */
 	ulint		src_line)/*!< in: line where the func invoked */
 	__attribute__((nonnull, warn_unused_result));
@@ -685,7 +678,7 @@ this function!
 A performance schema instrumented wrapper function for os_file_close().
 @return TRUE if success */
 UNIV_INLINE
-ibool
+bool
 pfs_os_file_close_func(
 /*===================*/
         os_file_t	file,	/*!< in, own: handle to a file */
@@ -696,9 +689,9 @@ NOTE! Please use the corresponding macro os_file_read(), not directly
 this function!
 This is the performance schema instrumented wrapper function for
 os_file_read() which requests a synchronous read operation.
-@return	TRUE if request was successful, FALSE if fail */
+@return TRUE if request was successful, FALSE if fail */
 UNIV_INLINE
-ibool
+bool
 pfs_os_file_read_func(
 /*==================*/
 	os_file_t	file,	/*!< in: handle to a file */
@@ -714,9 +707,9 @@ not directly this function!
 This is the performance schema instrumented wrapper function for
 os_file_read_no_error_handling_func() which requests a synchronous
 read operation.
-@return	TRUE if request was successful, FALSE if fail */
+@return TRUE if request was successful, FALSE if fail */
 UNIV_INLINE
-ibool
+bool
 pfs_os_file_read_no_error_handling_func(
 /*====================================*/
 	os_file_t	file,	/*!< in: handle to a file */
@@ -733,7 +726,7 @@ Performance schema wrapper function of os_aio() which requests
 an asynchronous i/o operation.
 @return TRUE if request was queued successfully, FALSE if fail */
 UNIV_INLINE
-ibool
+bool
 pfs_os_aio_func(
 /*============*/
 	ulint		type,	/*!< in: OS_FILE_READ or OS_FILE_WRITE */
@@ -745,6 +738,9 @@ pfs_os_aio_func(
 				to write */
 	os_offset_t	offset,	/*!< in: file offset where to read or write */
 	ulint		n,	/*!< in: number of bytes to read or write */
+	bool		read_only_mode,
+				/*!< in: if true, read only mode
+				checks are enforced. */
 	fil_node_t*	message1,/*!< in: message for the aio handler
 				(can be used to identify a completed
 				aio operation); ignored if mode is
@@ -760,9 +756,9 @@ NOTE! Please use the corresponding macro os_file_write(), not directly
 this function!
 This is the performance schema instrumented wrapper function for
 os_file_write() which requests a synchronous write operation.
-@return	TRUE if request was successful, FALSE if fail */
+@return TRUE if request was successful, FALSE if fail */
 UNIV_INLINE
-ibool
+bool
 pfs_os_file_write_func(
 /*===================*/
 	const char*	name,	/*!< in: name of the file or path as a
@@ -781,7 +777,7 @@ os_file_flush() which flushes the write buffers of a given file to the disk.
 Flushes the write buffers of a given file to the disk.
 @return TRUE if success */
 UNIV_INLINE
-ibool
+bool
 pfs_os_file_flush_func(
 /*===================*/
 	os_file_t	file,	/*!< in, own: handle to a file */
@@ -795,7 +791,7 @@ This is the performance schema instrumented wrapper function for
 os_file_rename()
 @return TRUE if success */
 UNIV_INLINE
-ibool
+bool
 pfs_os_file_rename_func(
 /*====================*/
 	mysql_pfs_key_t	key,	/*!< in: Performance Schema Key */
@@ -834,6 +830,7 @@ pfs_os_file_delete_if_exists_func(
 	mysql_pfs_key_t	key,	/*!< in: Performance Schema Key */
 	const char*	name,	/*!< in: old file path as a null-terminated
 				string */
+	bool*		exist,	/*!< out: indicate if file pre-exist */
 	const char*	src_file,/*!< in: file name where func invoked */
 	ulint		src_line);/*!< in: line where the func invoked */
 #endif	/* UNIV_PFS_IO */
@@ -841,17 +838,15 @@ pfs_os_file_delete_if_exists_func(
 #ifdef UNIV_HOTBACKUP
 /***********************************************************************//**
 Closes a file handle.
-@return	TRUE if success */
-UNIV_INTERN
-ibool
+@return TRUE if success */
+bool
 os_file_close_no_error_handling(
 /*============================*/
 	os_file_t	file);	/*!< in, own: handle to a file */
 #endif /* UNIV_HOTBACKUP */
 /***********************************************************************//**
 Gets a file size.
-@return	file size, or (os_offset_t) -1 on failure */
-UNIV_INTERN
+@return file size, or (os_offset_t) -1 on failure */
 os_offset_t
 os_file_get_size(
 /*=============*/
@@ -859,30 +854,40 @@ os_file_get_size(
 	__attribute__((warn_unused_result));
 /***********************************************************************//**
 Write the specified number of zeros to a newly created file.
-@return	TRUE if success */
-UNIV_INTERN
-ibool
+@return TRUE if success */
+bool
 os_file_set_size(
 /*=============*/
 	const char*	name,	/*!< in: name of the file or path as a
 				null-terminated string */
 	os_file_t	file,	/*!< in: handle to a file */
-	os_offset_t	size)	/*!< in: file size */
+	os_offset_t	size,	/*!< in: file size */
+	bool		read_only_mode)
+				/*!< in: if true, read only mode
+				checks are enforced. */
 	__attribute__((nonnull, warn_unused_result));
 /***********************************************************************//**
 Truncates a file at its current position.
-@return	TRUE if success */
-UNIV_INTERN
-ibool
+@return TRUE if success */
+bool
 os_file_set_eof(
 /*============*/
 	FILE*		file);	/*!< in: file to be truncated */
 /***********************************************************************//**
+Truncates a file to a specified size in bytes. Do nothing if the size
+preserved is smaller or equal than current size of file.
+@return true if success */
+bool
+os_file_truncate(
+/*=============*/
+	const char*	pathname,	/*!< in: file path */
+	os_file_t	file,		/*!< in: file to be truncated */
+	os_offset_t	size);		/*!< in: size preserved in bytes */
+/***********************************************************************//**
 NOTE! Use the corresponding macro os_file_flush(), not directly this function!
 Flushes the write buffers of a given file to the disk.
-@return	TRUE if success */
-UNIV_INTERN
-ibool
+@return TRUE if success */
+bool
 os_file_flush_func(
 /*===============*/
 	os_file_t	file);	/*!< in, own: handle to a file */
@@ -891,8 +896,7 @@ Retrieves the last error number if an error occurs in a file io function.
 The number should be retrieved before any other OS calls (because they may
 overwrite the error number). If the number is not known to this program,
 the OS error number + 100 is returned.
-@return	error number, or OS error number + 100 */
-UNIV_INTERN
+@return error number, or OS error number + 100 */
 ulint
 os_file_get_last_error(
 /*===================*/
@@ -901,9 +905,8 @@ os_file_get_last_error(
 /*******************************************************************//**
 NOTE! Use the corresponding macro os_file_read(), not directly this function!
 Requests a synchronous read operation.
-@return	TRUE if request was successful, FALSE if fail */
-UNIV_INTERN
-ibool
+@return TRUE if request was successful, FALSE if fail */
+bool
 os_file_read_func(
 /*==============*/
 	os_file_t	file,	/*!< in: handle to a file */
@@ -914,7 +917,6 @@ os_file_read_func(
 Rewind file to its start, read at most size - 1 bytes from it to str, and
 NUL-terminate str. All errors are silently ignored. This function is
 mostly meant to be used with temporary files. */
-UNIV_INTERN
 void
 os_file_read_string(
 /*================*/
@@ -926,9 +928,8 @@ NOTE! Use the corresponding macro os_file_read_no_error_handling(),
 not directly this function!
 Requests a synchronous positioned read operation. This function does not do
 any error handling. In case of error it returns FALSE.
-@return	TRUE if request was successful, FALSE if fail */
-UNIV_INTERN
-ibool
+@return TRUE if request was successful, FALSE if fail */
+bool
 os_file_read_no_error_handling_func(
 /*================================*/
 	os_file_t	file,	/*!< in: handle to a file */
@@ -940,9 +941,8 @@ os_file_read_no_error_handling_func(
 NOTE! Use the corresponding macro os_file_write(), not directly this
 function!
 Requests a synchronous write operation.
-@return	TRUE if request was successful, FALSE if fail */
-UNIV_INTERN
-ibool
+@return TRUE if request was successful, FALSE if fail */
+bool
 os_file_write_func(
 /*===============*/
 	const char*	name,	/*!< in: name of the file or path as a
@@ -953,13 +953,12 @@ os_file_write_func(
 	ulint		n);	/*!< in: number of bytes to write */
 /*******************************************************************//**
 Check the existence and type of the given file.
-@return	TRUE if call succeeded */
-UNIV_INTERN
-ibool
+@return TRUE if call succeeded */
+bool
 os_file_status(
 /*===========*/
 	const char*	path,	/*!< in:	pathname of the file */
-	ibool*		exists,	/*!< out: TRUE if file exists */
+	bool*		exists,	/*!< out: TRUE if file exists */
 	os_file_type_t* type);	/*!< out: type of the file (if it exists) */
 /****************************************************************//**
 The function os_file_dirname returns a directory component of a
@@ -988,8 +987,7 @@ returned by dirname and basename for different paths:
        "."	      "."	     "."
        ".."	      "."	     ".."
 
-@return	own: directory component of the pathname */
-UNIV_INTERN
+@return own: directory component of the pathname */
 char*
 os_file_dirname(
 /*============*/
@@ -1004,33 +1002,13 @@ the forward slash.  Both input strings are null terminated.
 This function allocates memory to be returned.  It is the callers
 responsibility to free the return value after it is no longer needed.
 
-@return	own: new full pathname */
-UNIV_INTERN
+@return own: new full pathname */
 char*
 os_file_make_new_pathname(
 /*======================*/
 	const char*	old_path,	/*!< in: pathname */
 	const char*	new_name);	/*!< in: new file name */
-/****************************************************************//**
-This function returns a remote path name by combining a data directory
-path provided in a DATA DIRECTORY clause with the tablename which is
-in the form 'database/tablename'.  It strips the file basename (which
-is the tablename) found after the last directory in the path provided.
-The full filepath created will include the database name as a directory
-under the path provided.  The filename is the tablename with the '.ibd'
-extension. All input and output strings are null-terminated.
 
-This function allocates memory to be returned.  It is the callers
-responsibility to free the return value after it is no longer needed.
-
-@return	own: A full pathname; data_dir_path/databasename/tablename.ibd */
-UNIV_INTERN
-char*
-os_file_make_remote_pathname(
-/*=========================*/
-	const char*	data_dir_path,	/*!< in: pathname */
-	const char*	tablename,	/*!< in: tablename */
-	const char*	extention);	/*!< in: file extention; ibd,cfg*/
 /****************************************************************//**
 This function reduces a null-terminated full remote path name into
 the path that is sent by MySQL for DATA DIRECTORY clause.  It replaces
@@ -1043,16 +1021,15 @@ This function manipulates that path in place.
 
 If the path format is not as expected, just return.  The result is used
 to inform a SHOW CREATE TABLE command. */
-UNIV_INTERN
 void
 os_file_make_data_dir_path(
 /*========================*/
 	char*	data_dir_path);	/*!< in/out: full path/data_dir_path */
+
 /****************************************************************//**
 Creates all missing subdirectories along the given path.
-@return	TRUE if call succeeded FALSE otherwise */
-UNIV_INTERN
-ibool
+@return TRUE if call succeeded FALSE otherwise */
+dberr_t
 os_file_create_subdirs_if_needed(
 /*=============================*/
 	const char*	path);	/*!< in: path name */
@@ -1063,8 +1040,7 @@ array is divided logically into n_read_segs and n_write_segs
 respectively. The caller must create an i/o handler thread for each
 segment in these arrays. This function also creates the sync array.
 No i/o handler thread needs to be created for that */
-UNIV_INTERN
-ibool
+bool
 os_aio_init(
 /*========*/
 	ulint	n_per_seg,	/*<! in: maximum number of pending aio
@@ -1075,7 +1051,6 @@ os_aio_init(
 				array */
 /***********************************************************************
 Frees the asynchronous io system. */
-UNIV_INTERN
 void
 os_aio_free(void);
 /*=============*/
@@ -1083,9 +1058,8 @@ os_aio_free(void);
 /*******************************************************************//**
 NOTE! Use the corresponding macro os_aio(), not directly this function!
 Requests an asynchronous i/o operation.
-@return	TRUE if request was queued successfully, FALSE if fail */
-UNIV_INTERN
-ibool
+@return TRUE if request was queued successfully, FALSE if fail */
+bool
 os_aio_func(
 /*========*/
 	ulint		type,	/*!< in: OS_FILE_READ or OS_FILE_WRITE */
@@ -1109,6 +1083,9 @@ os_aio_func(
 				to write */
 	os_offset_t	offset,	/*!< in: file offset where to read or write */
 	ulint		n,	/*!< in: number of bytes to read or write */
+	bool		read_only_mode,
+				/*!< in: if true, read only mode
+				checks are enforced. */
 	fil_node_t*	message1,/*!< in: message for the aio handler
 				(can be used to identify a completed
 				aio operation); ignored if mode is
@@ -1120,20 +1097,17 @@ os_aio_func(
 /************************************************************************//**
 Wakes up all async i/o threads so that they know to exit themselves in
 shutdown. */
-UNIV_INTERN
 void
 os_aio_wake_all_threads_at_shutdown(void);
 /*=====================================*/
 /************************************************************************//**
 Waits until there are no pending writes in os_aio_write_array. There can
 be other, synchronous, pending writes. */
-UNIV_INTERN
 void
 os_aio_wait_until_no_pending_writes(void);
 /*=====================================*/
 /**********************************************************************//**
 Wakes up simulated aio i/o-handler threads if they have something to do. */
-UNIV_INTERN
 void
 os_aio_simulated_wake_handler_threads(void);
 /*=======================================*/
@@ -1142,7 +1116,6 @@ This function can be called if one wants to post a batch of reads and
 prefers an i/o-handler thread to handle them all at once later. You must
 call os_aio_simulated_wake_handler_threads later to ensure the threads
 are not left sleeping! */
-UNIV_INTERN
 void
 os_aio_simulated_put_read_threads_to_sleep(void);
 /*============================================*/
@@ -1155,9 +1128,8 @@ for completed requests. The aio array of pending requests is divided
 into segments. The thread specifies which segment or slot it wants to wait
 for. NOTE: this function will also take care of freeing the aio slot,
 therefore no other thread is allowed to do the freeing!
-@return	TRUE if the aio operation succeeded */
-UNIV_INTERN
-ibool
+@return TRUE if the aio operation succeeded */
+bool
 os_aio_windows_handle(
 /*==================*/
 	ulint	segment,	/*!< in: the number of the segment in the aio
@@ -1182,9 +1154,8 @@ os_aio_windows_handle(
 /**********************************************************************//**
 Does simulated aio. This function should be called by an i/o-handler
 thread.
-@return	TRUE if the aio operation succeeded */
-UNIV_INTERN
-ibool
+@return TRUE if the aio operation succeeded */
+bool
 os_aio_simulated_handle(
 /*====================*/
 	ulint	segment,	/*!< in: the number of the segment in the aio
@@ -1201,54 +1172,57 @@ os_aio_simulated_handle(
 	ulint*	type);		/*!< out: OS_FILE_WRITE or ..._READ */
 /**********************************************************************//**
 Validates the consistency of the aio system.
-@return	TRUE if ok */
-UNIV_INTERN
-ibool
+@return TRUE if ok */
+bool
 os_aio_validate(void);
 /*=================*/
 /**********************************************************************//**
 Prints info of the aio arrays. */
-UNIV_INTERN
 void
 os_aio_print(
 /*=========*/
 	FILE*	file);	/*!< in: file where to print */
 /**********************************************************************//**
 Refreshes the statistics used to print per-second averages. */
-UNIV_INTERN
 void
 os_aio_refresh_stats(void);
 /*======================*/
 
-#ifdef UNIV_DEBUG
 /**********************************************************************//**
 Checks that all slots in the system have been freed, that is, there are
 no pending io operations. */
-UNIV_INTERN
-ibool
+bool
 os_aio_all_slots_free(void);
 /*=======================*/
+
+#ifdef UNIV_DEBUG
+/** Prints all pending IO
+@param[in]	file	file where to print */
+void
+os_aio_print_pending_io(
+	FILE*	file);
 #endif /* UNIV_DEBUG */
 
 /*******************************************************************//**
 This function returns information about the specified file
-@return	DB_SUCCESS if all OK */
-UNIV_INTERN
+@return DB_SUCCESS if all OK */
 dberr_t
 os_file_get_status(
 /*===============*/
 	const char*	path,		/*!< in: pathname of the file */
 	os_file_stat_t* stat_info,	/*!< information of a file in a
 					directory */
-	bool		check_rw_perm);	/*!< in: for testing whether the
+	bool		check_rw_perm,	/*!< in: for testing whether the
 					file can be opened in RW mode */
+	bool		read_only_mode);
+					/*!< in: if true read only mode
+					checks are enforced. */
 
 #if !defined(UNIV_HOTBACKUP)
 /*********************************************************************//**
 Creates a temporary file that will be deleted on close.
 This function is defined in ha_innodb.cc.
-@return	temporary file descriptor, or < 0 on error */
-UNIV_INTERN
+@return temporary file descriptor, or < 0 on error */
 int
 innobase_mysql_tmpfile(void);
 /*========================*/
@@ -1263,9 +1237,8 @@ for completed requests. The aio array of pending requests is divided
 into segments. The thread specifies which segment or slot it wants to wait
 for. NOTE: this function will also take care of freeing the aio slot,
 therefore no other thread is allowed to do the freeing!
-@return	TRUE if the IO was successful */
-UNIV_INTERN
-ibool
+@return TRUE if the IO was successful */
+bool
 os_aio_linux_handle(
 /*================*/
 	ulint	global_seg,	/*!< in: segment number in the aio array
@@ -1282,6 +1255,13 @@ os_aio_linux_handle(
 	ulint*	type);		/*!< out: OS_FILE_WRITE or ..._READ */
 #endif /* LINUX_NATIVE_AIO */
 
+/** Normalizes a directory path for Windows: converts '/' to '\'.
+@param[in,out] str A null-terminated Windows directory and file path */
+#ifdef _WIN32
+void os_normalize_path_for_win(char*	str);
+#else
+#define os_normalize_path_for_win(str)
+#endif
 #ifndef UNIV_NONINL
 #include "os0file.ic"
 #endif
