@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -83,18 +83,16 @@ int mi_lock_database(MI_INFO *info, int lock_type)
 			   (uint) share->changed, share->w_locks));
 	if (share->changed && !share->w_locks)
 	{
-#ifdef HAVE_MMAP
-    if ((info->s->mmaped_length != info->s->state.state.data_file_length) &&
-        (info->s->nonmmaped_inserts > MAX_NONMAPPED_INSERTS))
-    {
-      if (info->s->concurrent_insert)
-        mysql_rwlock_wrlock(&info->s->mmap_lock);
-      mi_remap_file(info, info->s->state.state.data_file_length);
-      info->s->nonmmaped_inserts= 0;
-      if (info->s->concurrent_insert)
-        mysql_rwlock_unlock(&info->s->mmap_lock);
-    }
-#endif
+          if ((info->s->mmaped_length != info->s->state.state.data_file_length) &&
+              (info->s->nonmmaped_inserts > MAX_NONMAPPED_INSERTS))
+          {
+            if (info->s->concurrent_insert)
+              mysql_rwlock_wrlock(&info->s->mmap_lock);
+            mi_remap_file(info, info->s->state.state.data_file_length);
+            info->s->nonmmaped_inserts= 0;
+            if (info->s->concurrent_insert)
+              mysql_rwlock_unlock(&info->s->mmap_lock);
+          }
 	  share->state.process= share->last_process=share->this_process;
 	  share->state.unique=   info->last_unique=  info->this_unique;
 	  share->state.update_count= info->last_loop= ++info->this_loop;
@@ -103,8 +101,6 @@ int mi_lock_database(MI_INFO *info, int lock_type)
 	  share->changed=0;
 	  if (myisam_flush)
 	  {
-            if (share->file_map)
-              my_msync(info->dfile, share->file_map, share->mmaped_length, MS_SYNC);
             if (mysql_file_sync(share->kfile, MYF(0)))
 	      error= my_errno;
             if (mysql_file_sync(info->dfile, MYF(0)))
@@ -229,10 +225,6 @@ int mi_lock_database(MI_INFO *info, int lock_type)
       info->invalidator=info->s->invalidator;
       share->w_locks++;
       share->tot_locks++;
-
-      DBUG_EXECUTE_IF("simulate_incorrect_share_wlock_value",
-                      DEBUG_SYNC_C("after_share_wlock_increment"););
-
       info->s->in_use= list_add(info->s->in_use, &info->in_use);
       break;
     default:
@@ -395,7 +387,7 @@ my_bool mi_check_status(void *param)
  ** functions to read / write the state
 ****************************************************************************/
 
-int _mi_readinfo(register MI_INFO *info, int lock_type, int check_keybuffer)
+int _mi_readinfo(MI_INFO *info, int lock_type, int check_keybuffer)
 {
   DBUG_ENTER("_mi_readinfo");
 
@@ -434,7 +426,7 @@ int _mi_readinfo(register MI_INFO *info, int lock_type, int check_keybuffer)
   request
 */
 
-int _mi_writeinfo(register MI_INFO *info, uint operation)
+int _mi_writeinfo(MI_INFO *info, uint operation)
 {
   int error,olderror;
   MYISAM_SHARE *share=info->s;
@@ -456,8 +448,6 @@ int _mi_writeinfo(register MI_INFO *info, uint operation)
 #ifdef _WIN32
       if (myisam_flush)
       {
-       if (share->file_map)
-         my_msync(info->dfile, share->file_map, share->mmaped_length, MS_SYNC);
         mysql_file_sync(share->kfile, 0);
         mysql_file_sync(info->dfile, 0);
       }
@@ -478,7 +468,7 @@ int _mi_writeinfo(register MI_INFO *info, uint operation)
 	/* Test if someone has changed the database */
 	/* (Should be called after readinfo) */
 
-int _mi_test_if_changed(register MI_INFO *info)
+int _mi_test_if_changed(MI_INFO *info)
 {
   MYISAM_SHARE *share=info->s;
   if (share->state.process != share->last_process ||
@@ -523,7 +513,7 @@ int _mi_test_if_changed(register MI_INFO *info)
 int _mi_mark_file_changed(MI_INFO *info)
 {
   uchar buff[3];
-  register MYISAM_SHARE *share=info->s;
+  MYISAM_SHARE *share=info->s;
   DBUG_ENTER("_mi_mark_file_changed");
 
   if (!(share->state.changed & STATE_CHANGED) || ! share->global_changed)
@@ -556,7 +546,7 @@ int _mi_mark_file_changed(MI_INFO *info)
 int _mi_decrement_open_count(MI_INFO *info)
 {
   uchar buff[2];
-  register MYISAM_SHARE *share=info->s;
+  MYISAM_SHARE *share=info->s;
   int lock_error=0,write_error=0;
   if (share->global_changed)
   {
