@@ -1,21 +1,14 @@
 /*
-   Copyright (c) 2004, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2004, 2014, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -51,7 +44,7 @@ int
 main(int argc, char** argv){
   NDB_INIT(argv[0]);
   const char *load_default_groups[]= { "mysql_cluster",0 };
-  load_defaults("my",load_default_groups,&argc,&argv);
+  ndb_load_defaults(NULL, load_default_groups,&argc,&argv);
   int ho_error;
 
   if ((ho_error=handle_options(&argc, &argv, my_long_options,
@@ -166,7 +159,7 @@ create_random_table(Ndb* pNdb)
       memset(defbuf, 0, (length + 7)/8);
       for (Uint32 j = 0; j < len/8; j++)
         defbuf[j] = 0x63;
-      col.setDefaultValue(defbuf, (len + 7)/8);
+      col.setDefaultValue(defbuf, 4*((len + 31)/32));
       col.setLength(len); length -= len;
       int nullable = (rand() >> 16) & 1;
       col.setNullable(nullable); length -= nullable;
@@ -504,7 +497,7 @@ testRanges(Uint32 bitmask_size)
       // 1) Look up allocation
       // 2) Check data
       // 3) free it
-      size_t j;
+      unsigned j;
       Uint32 min, max;
       for(j = 0; j<alloc_list.size(); j++)
       {
@@ -530,7 +523,7 @@ testRanges(Uint32 bitmask_size)
 	print(tmp.getBase(), max - min);
 
 	printf(" save: ");
-	size_t k;
+        unsigned k;
 	Alloc& a = alloc_list[j];
 	for(k = 0; k<a.data.size(); k++)
 	  printf("%.8x ", a.data[k]);
@@ -568,7 +561,7 @@ testRanges(Uint32 bitmask_size)
       a.data.fill(((sz+31)>> 5)-1, zero);
       if(BITMASK_DEBUG)
 	printf("pos %d -> alloc [ %d %d ]", pos, pos, pos+sz);
-      for(size_t j = 0; j<sz; j++)
+      for(Uint32 j = 0; j<sz; j++)
       {
 	BitmaskImpl::set(sz32, alloc_mask.getBase(), pos+j);
 	if((lrand() % 1000) > 500)
